//
//	Axis205.h	This file is a part of the IKAROS project
//					A module to grab images and control movements of a network camera
//
//    Copyright (C) 2002  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#ifndef AXIS205
#define AXIS205

#include "IKAROS.h"

#ifdef USE_SOCKET
#ifdef USE_LIBJPEG

#define BUFFER_SIZE 10000

class Axis205: public Module 
{
public:

	Axis205(char * name, Parameter * p);
	virtual ~Axis205();

	static Module * Create(char * name, Parameter * p);

	void Init();
	void Tick();	
    
private:
    enum            {aJPEGType = 0, anMJPEGType};   // JPEG/MJPG
	int				type;	// 0 = JPG, 1 = MJPG
	int				grab;	// 0 = sync, 1 = async
	
	int				size_x;	// Size of the image to grab
	int				size_y;

	float *			intensity;
	float *			red;
	float *			green;
	float *			blue;

	char *			host_ip;
	int				host_port;
	int				socket_descriptor;
	
	char *			image_request;
	char            myMessage[256];
	
    // Reduce sampling rate by dividing the master clock
	int				mySampleInterval;
	int				myTickCounter;

    // Skip myTransferMultiplier - 1 images from the camera to reduce the send buffer latency
    int             myTransferMultiplier;

	char            myRecvBuf[BUFFER_SIZE];
	unsigned int    myRecvCount;
    unsigned int    myCurRecvBufIndex;

	void	Grab();

    int getImage(char * hostname, int port, char * request, int & sd, char * result);
    static int die(char * msg);
    bool skip(int sd, const char * theSearchString);
    void readBuffer(int sd);

    static int requestData(char * hostname, int port, char * request);	// return socket descriptor

	void	DecodeImage(unsigned char * source);
};

#endif
#endif

#ifndef USE_SOCKET
#ifndef USE_LIBJPEG
class Axis205 {
public:
	static Module * Create(char * name, Parameter * p) { return NULL; }
};
#endif
#endif

#endif
