//
//	NetworkCamera.cc	This file is a part of the IKAROS project
//					A module to grab images and control movements of a network camera
//
//    Copyright (C) 2002  Christian Balkenius
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

// NOTE: This code looks as a mess since it glues together code uding three different coding styles:
// libjpeg, Ikaros, and the POSIX socket library.

#include "NetworkCamera.h"

#ifdef USE_LIBJPEG
#ifdef USE_SOCKET

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>

#define MAX_STR_LEN 512



// #define MAXDATASIZE		2560000L 		// max number of bytes we can get at once

extern "C" {
#include "jpeglib.h"
//#include "jinclude.h"
//#include "jerror.h"
}

#include <setjmp.h>


static int
die(char * msg)
{
	printf(msg);
	exit(1);
	return 1;
}



int
getData(char * hostname, int port, char * request, char * result, char * buffer)
{
	
	int i, sd, rc, size, dst=0;
	struct sockaddr_in localAddr, servAddr;
	struct hostent *h;

	(h = gethostbyname(hostname)) != NULL || die("unknown host\n");

	servAddr.sin_family = h->h_addrtype;
	memcpy((char *) &servAddr.sin_addr.s_addr, h->h_addr_list[0], h->h_length);
	servAddr.sin_port = htons(port);

	(sd = socket(AF_INET, SOCK_STREAM, 0)) >= 0 || die("cannot open socket\n"); 

	localAddr.sin_family = AF_INET;	
	localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	localAddr.sin_port = htons(0);
  
	(rc = bind(sd, (struct sockaddr *) &localAddr, sizeof(localAddr))) >= 0 || die("cannot open port\n");
	(rc = connect(sd, (struct sockaddr *) &servAddr, sizeof(servAddr))) >= 0 || die("cannot connect\n");
	(rc = write(sd, request, strlen(request))) >= 0 || die("cannot send data\n");

	rc = 0, size = 0;
	do
   	{
		memset(buffer, 0x0, BUFFER_SIZE);    //  init line 
		rc = read(sd, buffer, BUFFER_SIZE);
		if(rc > 0)
		{
			for(i=0; i<rc; i++)
				result[dst++] = buffer[i];
			size += rc;
		}
	} while(rc>0);

	result[dst++] = '\0';

	close(sd);
	
	return size;
}



int
requestData(char * hostname, int port, char * request)	// return socket descriptor
{
	int sd, rc;
	struct sockaddr_in localAddr, servAddr;
	struct hostent *h;

	(h = gethostbyname(hostname)) != NULL || die("unknown host\n");

	servAddr.sin_family = h->h_addrtype;
	memcpy((char *) &servAddr.sin_addr.s_addr, h->h_addr_list[0], h->h_length);
	servAddr.sin_port = htons(port);

	(sd = socket(AF_INET, SOCK_STREAM, 0)) >= 0 || die("cannot open socket\n"); 

	localAddr.sin_family = AF_INET;	
	localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	localAddr.sin_port = htons(0);
  
	(rc = bind(sd, (struct sockaddr *) &localAddr, sizeof(localAddr))) >= 0 || die("cannot open port\n");
	(rc = connect(sd, (struct sockaddr *) &servAddr, sizeof(servAddr))) >= 0 || die("cannot connect\n");
	(rc = write(sd, request, strlen(request))) >= 0 || die("cannot send data\n");

	return sd;
}




int
readData(int sd, char * result, char * buffer)
{
	int i, rc, size, dst=0;

	rc = 0, size = 0;
	do
   	{
		memset(buffer, 0x0, BUFFER_SIZE);    //  init line 
		rc = read(sd, buffer, BUFFER_SIZE);
		if(rc > 0)
		{
			for(i=0; i<rc; i++)
				result[dst++] = buffer[i];
			size += rc;
		}
	} while(rc>0);

	result[dst++] = '\0';

	close(sd);
	return size;
}



Module *
NetworkCamera::Create(char * name, Parameter * p)
{
	return new NetworkCamera(name, p);
}



NetworkCamera::NetworkCamera(char * name, Parameter * p):
	Module(name)
{
	intensity	=	NULL;
	red		=	NULL;
	green	=	NULL;
	blue		=	NULL;
	
	type	= GetIntValueFromList(p, "type", "JPEG/MJPG");
	grab	= GetIntValueFromList(p, "grab", "synchronous/asynchronous");
	
	size_x  = GetIntValue(p, "size_x", 352);
	size_y  = GetIntValue(p, "size_y", 240);

//	printf("GRAB SIZE: %d, %d\n", size_x, size_y);
                
	if(size_x*size_y < 1)
	{
		Notify(msg_fatal_error, "Image size not supplied.\n");
		return;
	}
	
	socket_descriptor	=	-1;

	host_ip			=	GetValue(p, "host_ip");
	host_port			=	GetIntValue(p, "host_port", 80);	
	image_request		=	GetValue(p, "image_request");

	interval			=	GetIntValue(p, "interval", 1);
	counter			=	0;

	AddOutput("INTENSITY", size_x, size_y);
	AddOutput("RED", size_x, size_y);
	AddOutput("GREEN", size_x, size_y);
	AddOutput("BLUE", size_x, size_y);
}



NetworkCamera::~NetworkCamera()
{

}



void
NetworkCamera::Init()
{
	intensity			= GetOutputArray("INTENSITY");
	red                                            = GetOutputArray("RED");
	green			= GetOutputArray("GREEN");
	blue                                          = GetOutputArray("BLUE");
}



//
// IN MEMORY CONVERSION DATA SOURCE MANAGER
//


struct my_error_mgr {
  struct jpeg_error_mgr pub;	// "public" fields 
  jmp_buf setjmp_buffer;		// for return to caller
};



typedef struct my_error_mgr * my_error_ptr;



static void
my_error_exit (j_common_ptr cinfo)
{
  my_error_ptr myerr = (my_error_ptr) cinfo->err;
  (*cinfo->err->output_message) (cinfo);
  longjmp(myerr->setjmp_buffer, 1);
}



void
init_source (j_decompress_ptr cinfo)
{
}



boolean
fill_input_buffer (j_decompress_ptr cinfo)
{
	return TRUE;
}



void
skip_input_data (j_decompress_ptr cinfo, long num_bytes)
{
	cinfo->src->next_input_byte += (size_t) num_bytes;
}



void
term_source (j_decompress_ptr cinfo)
{
}



void
NetworkCamera::DecodeImage(unsigned char * source)
{
	struct jpeg_decompress_struct cinfo;
	struct my_error_mgr jerr;
	struct jpeg_source_mgr * src;

	JSAMPARRAY buffer;		// Output row buffer 
	int row_stride;			// physical row width in output buffer 
	
	// BUILD ERROR HANDLER

	cinfo.err = jpeg_std_error(&jerr.pub);
	jerr.pub.error_exit = my_error_exit;

	if (setjmp(jerr.setjmp_buffer))
	{
		jpeg_destroy_decompress(&cinfo);
		return;	// REPORT ERRORs
	}

	jpeg_create_decompress(&cinfo);
	
	// BUILD SOURCE MANAGER
	
	if (cinfo.src == NULL)	/* first time for this JPEG object? */
		cinfo.src = (struct jpeg_source_mgr *)(*cinfo.mem->alloc_small) ((j_common_ptr)(&cinfo),
		JPOOL_PERMANENT, sizeof(struct jpeg_source_mgr));   // WAS "SIZEOF"
	
	src = (struct jpeg_source_mgr *) cinfo.src;

	src->init_source = init_source;
	src->fill_input_buffer = fill_input_buffer;
	src->skip_input_data = skip_input_data;
	src->resync_to_restart = jpeg_resync_to_restart; /* use default method */
	src->term_source = term_source;
 	src->bytes_in_buffer = (size_t)(-1);
	src->next_input_byte = (JOCTET *)(source);	// Decode this data!!!
	
	// START DECODING THE JPEG IMAGE
	
	(void) jpeg_read_header(&cinfo, TRUE);

	cinfo.scale_denom = 1; // 4; 	/////// TEST TEST TEST TEST TEST

	(void) jpeg_start_decompress(&cinfo);
	
	row_stride = cinfo.output_width * cinfo.output_components;

//	printf("# width = %d height = %d components = %d\n", cinfo.output_width, cinfo.output_height, cinfo.output_components);
  
//	*w = cinfo.output_width;
//	*h = cinfo.output_height;
  
	buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

	if(cinfo.output_components == 1) // Gray Scale Image
            {
		while (cinfo.output_scanline < cinfo.output_height)
		{
		//	printf("scanline = %d\n", cinfo.output_scanline);
			(void) jpeg_read_scanlines(&cinfo, buffer, 1);
			for(int i=0; i<size_x; i++) // CHECK BOUNDS
			{
				int ix = size_x*(cinfo.output_scanline-1) + i;
				intensity[ix]	= float(buffer[0][i]);
				red[ix]		= intensity[ix];
				green[ix]		= intensity[ix];
				blue[ix]		= intensity[ix];
			}
		}
	
        }
	else // RGB Color Image
	{
		while (cinfo.output_scanline < cinfo.output_height)
		{
		//	printf("scanline = %d\n", cinfo.output_scanline);
			(void) jpeg_read_scanlines(&cinfo, buffer, 1);
			int j = 0;
			for(int i=0; i<size_x; i++) // CHECK BOUNDS
			{
				int ix = size_x*(cinfo.output_scanline-1) + i;
				red[ix]		= float(buffer[0][j++]);
				green[ix]		= float(buffer[0][j++]);
				blue[ix]		= float(buffer[0][j++]);
				intensity[ix]	= (red[ix] + green[ix] +blue[ix]) / 3.0;	// Do this correctly later!!!
			}
		}
	}

	(void) jpeg_finish_decompress(&cinfo);
	
	// printf("offset = %d\n", src->next_input_byte-source);
	
	jpeg_destroy_decompress(&cinfo);
}



void
NetworkCamera::SendCommand()
{
}



void
NetworkCamera::Grab()
{
    // Changed 100000 to 200000 050508 MA
	char buf[200000];	// This should be created and expanded dynamically
	char msg[256];
	int size, start = 0;

	sprintf(msg, "GET %s HTTP/1.1\r\nHOST: %s\r\n\r\n", image_request, host_ip);

	// Send GRAB to camera and receive data synchronously
	
	if(type == 0)
	{
		size = getData(host_ip, host_port, msg, buf, buffer);
		while(buf[start] != '\r' || buf[start+1] != '\n' || buf[start+2] != '\r' || buf[start+3] != '\n') // Skip ahead to image
			start++;
		DecodeImage((unsigned char *)(&buf[start+4]));
	}
	
	else
	{
		if(socket_descriptor != -1)
			size = readData(socket_descriptor, buf, buffer);
		socket_descriptor = requestData(host_ip, host_port, msg);
		while(buf[start] != '\r' || buf[start+1] != '\n' || buf[start+2] != '\r' || buf[start+3] != '\n') // Skip ahead to image
			start++;
		DecodeImage((unsigned char *)(&buf[start+4]));
	}
}



void
NetworkCamera::Tick()
{
	counter--;
	if(counter > 0)
                        return;
	
//	printf("Grabbing\n");
	Grab();
        
	counter = interval;
}

#endif
#endif

