//
//	VisualAttentionTest.h		This file is a part of the IKAROS project
// 				<Short description of the module>
//
//    Copyright (C) 2005 Mikael Asker
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	Created: 050429
//
//	<Additional description of the module>

#ifndef VISUAL_ATTENTION_TEST
#define VISUAL_ATTENTION_TEST

#include "IKAROS.h"
#include "KalmanFilter.h"
#include "Point2D.h"
#include "VisualCategories.h"

        
class TRMProxy
{
public:
        TRMProxy();
                
        void init(unsigned int  theImageWidth,
                  unsigned int  theImageHeight,
                  float**       theOutputMatrix);
                
        void reduce(unsigned int x, unsigned int y);
        void move(unsigned int x, unsigned int y);
        void enlarge(unsigned int x, unsigned int y);
        bool isCompletelyOpen() const {return iAmCompletelyOpen;}
        void open();

        virtual ~TRMProxy() {}

private:
        enum {aMapScale = 4, 
              aMaximumDivisor = 7, 
              aMinimumDivisor = 3, 
              aFullRelevance = 1};
                
        float** myOutputMatrix;   // Ikaros output matrix
        unsigned int myImageWidth;
        unsigned int myImageHeight;
        unsigned int myMapWidth;
        unsigned int myMapHeight;
        float myMinimumStdDev;
        float myMaximumStdDev;
                
        bool iAmCompletelyOpen;
        float myCurrentStdDev;
        float myCurrentVariance;
                
        void generateOutput(unsigned int x, unsigned int y);
        float weight(unsigned int x, unsigned int y, unsigned int i, unsigned int j);
};


class VisualAttentionTest: public Module 
{
public:

        VisualAttentionTest(char* theName, Parameter* theParameters);
        virtual ~VisualAttentionTest() {}
        
        static Module*  Create(char* theName, Parameter* theParameters);

        void		SetSizes();
        
        void 		Init();
        void 		Tick();

private:

        unsigned int myImageWidth;
        unsigned int myImageHeight;

        typedef enum {aLearnTask = 0, aSearchTask, aDetectTask, aFollowTask, anUndefinedTask} Task;
        Task myTask;        

        enum {aNotACoordinateValue = -1};
        Point2D myLearningCoordinates;
        
        int myLearningRadius;
        
        VisualCategories::CategoryIndex mySearchCategory;
        VisualCategories::CategoryIndex myDetectCategory;
        
        VisualCategories::CategoryIndex myFollowCategory;
        enum {aDetectingState = 0, aFollowingState, aSearchingState} myFollowState;
        enum {aProcessNoiseStdDev = 1, aMeasurementNoiseStdDev = 100};
        KalmanFilter myXTracker; 
        KalmanFilter myYTracker;
        
        TRMProxy myTRM;
        
        const bool myBiasFlag;

        // Declare some variables.

        // The input and output vectors. The storage is declared and handled by
        // Ikaros so you do not have to.

        float** myFOAInput;         // Focus of attention coordinates
        float** myCategoryInput;    // Detected category

        float** myCategoryOutput;   // Category to look for
        float** myIOROutput;        // Enable/disable IOR in VFP
        float** myLearnOutput;      // Learn a new category

        unsigned int myTickCounter;

        float myLastXPos;
        float myLastYPos;
        
        enum {anEnlargeTime = 0};
        unsigned int myEnlargeDelay;


        static bool learningTarget(const char *    theTargetString,
                                   const char * &  theErrorMessage,
                                   Point2D &       theTargetCoordinates,
                                   int&            theTargetRadius);

        void startTrackers();

        enum {aDelay = 2};
        void estimatePosition(float& theXPosEstimate, float& theYPosEstimate);
};

#endif
