//
//	VisualCategories.h		This file is a part of the IKAROS project
// 				<Short description of the module>
//
//    Copyright (C) 2005 Mikael Asker
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	Created: 050427
//
//	<Additional description of the module>

#ifndef VISUAL_CATEGORIES
#define VISUAL_CATEGORIES

#include "VisualFeaturesAndPositions.h"


class CategorySet
{
public:

        typedef int CategoryIdentifier;
        
        inline static bool compareFeature(int theFeatureIndex)
          {return (theFeatureIndex <= 11) || (18 <= theFeatureIndex && theFeatureIndex <= 47);}
          //  0-11 Color
          // 12-17 Flicker - excluded
          // 18-23 Intensity
          // 24-47 Orientation
          // 48-71 Motion - excluded

        CategorySet(const char * theFileName);
        CategoryIdentifier newCategory(const VisualFeaturesAndPositions::FeatureValueVector&);

        CategoryIdentifier categorizeFeatures(const VisualFeaturesAndPositions::FeatureValueVector&,
                                              unsigned theTickCounter,
                                              Module* theIkarosModule) const;

        VisualFeaturesAndPositions::FeatureValueVector featuresOfCategory(CategoryIdentifier) const;
        virtual ~CategorySet();

private:

        enum {aFeatureDistanceThreshold = 140};

        const string myFileName;

        typedef vector<VisualFeaturesAndPositions::FeatureValueVector> FeatureValueVectorVector;
        FeatureValueVectorVector myVectors;

        static float featureDistance(const VisualFeaturesAndPositions::FeatureValueVector&,
                                     const VisualFeaturesAndPositions::FeatureValueVector&);

        void save();
};


class VisualCategories: public Module 
{
public:

        typedef float CategoryIndex;
        enum {aNotACategoryIdentifier = -1};

        VisualCategories(char* theName, Parameter* theParameters);
        virtual ~VisualCategories();
        
        static Module*  Create(char* theName, Parameter* theParameters);

        void		SetSizes();
        
        void 		Init();
        void 		Tick();

private:
        
        // Declare some variables.
        
        CategorySet myCategorySet;

        // The input and output vectors. The storage is declared and handled by
        // Ikaros so you do not have to.

        float** myCategoryInput;        // Category -> bias
        float** myFeatureValuesInput;   // Features -> category
        float** myLearnInput;           // Features -> category

        float** myFilterBiasOutput;     // Category -> bias
        float** myCategoryOutput;       // Features -> category
        
        unsigned int myTickCounter;
        float myLastLearnInputValue;
};

#endif
