/*!@file ChannelsBeo.H Definitions for the Beo version of the channels */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: ChannelsBeo.H 2493 2003-04-29 08:42:03Z itti $
//

#ifndef CHANNELSBEO_H_DEFINED
#define CHANNELSBEO_H_DEFINED

#include "saliency.H"
#include "SingleChannelBeo.H"
#include "Channels.H"

// ######################################################################
// ######################################################################
// #################### SINGLE CHANNELS
// ######################################################################
// ######################################################################

// ######################################################################
//! Beowulf version of IntensityChannel
class IntensityChannelBeo : public IntensityChannel, public SingleChannelBeo
{
public:
  IntensityChannelBeo(ModelManager& mgr, const SharedPtr<Beowulf>& beow);
  virtual ~IntensityChannelBeo();
  virtual bool outputAvailable() const;
protected:
  virtual void doInput(const Image<PixRGB<float> >*,
                       const Image<float>* bwimg, const double t,
                       const Image<byte>& clipMask);
  virtual void start1();
};

// ######################################################################
//! Beowulf version of RedGreenChannel
class RedGreenChannelBeo : public RedGreenChannel, public SingleChannelBeo
{
public:
  RedGreenChannelBeo(ModelManager& mgr, const SharedPtr<Beowulf>& beow);
  virtual ~RedGreenChannelBeo();
  virtual bool outputAvailable() const;
protected:
  virtual void doInput(const Image<PixRGB<float> >*,
                       const Image<float>* bwimg, const double t,
                       const Image<byte>& clipMask);
  virtual void start1();
};

// ######################################################################
//! Beowulf version of BlueYellowChannel
class BlueYellowChannelBeo : public BlueYellowChannel, public SingleChannelBeo
{
public:
  BlueYellowChannelBeo(ModelManager& mgr, const SharedPtr<Beowulf>& beow);
  virtual ~BlueYellowChannelBeo();
  virtual bool outputAvailable() const;
protected:
  virtual void doInput(const Image<PixRGB<float> >*,
                       const Image<float>* bwimg, const double t,
                       const Image<byte>& clipMask);
  virtual void start1();
};

// ######################################################################
//! Beowulf version of GaborChannel
class GaborChannelBeo : public GaborChannel, public SingleChannelBeo
{
public:
  GaborChannelBeo(ModelManager& mgr, const uint oriIndex, const double ang,
                  const SharedPtr<Beowulf>& beow);
  virtual ~GaborChannelBeo();
  virtual bool outputAvailable() const;
protected:
  virtual void doInput(const Image<PixRGB<float> >*,
                       const Image<float>* bwimg, const double t,
                       const Image<byte>& clipMask);
  virtual void start1();
};

// ######################################################################
//! Beowulf version of FlickerChannel
class FlickerChannelBeo : public FlickerChannel, public SingleChannelBeo
{
public:
  FlickerChannelBeo(ModelManager& mgr, const SharedPtr<Beowulf>& beow);
  virtual ~FlickerChannelBeo();
  virtual bool outputAvailable() const;
protected:
  virtual void doInput(const Image<PixRGB<float> >*,
                       const Image<float>* bwimg, const double t,
                       const Image<byte>& clipMask);
  virtual void start1();
};

// ######################################################################
//! Beowulf version of ColorBandChannel
class ColorBandChannelBeo : public ColorBandChannel, public SingleChannelBeo
{
public:
  ColorBandChannelBeo(ModelManager& mgr, const uint bandidx,
                      const SharedPtr<Beowulf>& beow);
  virtual ~ColorBandChannelBeo();
  virtual bool outputAvailable() const;
protected:
  virtual void doInput(const Image<PixRGB<float> >*,
                       const Image<float>* bwimg, const double t,
                       const Image<byte>& clipMask);
  virtual void start1();
};

// ######################################################################
//! Beowulf version of DirectionChannel
class DirectionChannelBeo : public DirectionChannel, public SingleChannelBeo
{
public:
  DirectionChannelBeo(ModelManager& mgr, const uint dirIndex,
                      const double direction, const PyramidType type,
                      const SharedPtr<Beowulf>& beow);
  virtual ~DirectionChannelBeo();
  virtual bool outputAvailable() const;
protected:
  virtual void doInput(const Image<PixRGB<float> >*,
                       const Image<float>* bwimg, const double t,
                       const Image<byte>& clipMask);
  virtual void start1();
};

// ######################################################################
// ######################################################################
// #################### COMPLEX CHANNELS
// ######################################################################
// ######################################################################

// ######################################################################
//! Beowulf version of ColorChannel
class ColorChannelBeo : public ColorChannel
{
public:
  ColorChannelBeo(ModelManager& mgr, const SharedPtr<Beowulf>& beow);
  virtual ~ColorChannelBeo();
};

// ######################################################################
//! Beowulf version of OrientationChannel
class OrientationChannelBeo : public OrientationChannel
{
public:
  OrientationChannelBeo(ModelManager& mgr, const SharedPtr<Beowulf>& beow);
  virtual ~OrientationChannelBeo();
protected:
  virtual void buildSubChans();
private:
  SharedPtr<Beowulf> itsBeo; // keep track of beo for use in buildSubChans()
};

// ######################################################################
//! Beowulf version of MultiColorBandChannel
class MultiColorBandChannelBeo : public MultiColorBandChannel
{
public:
  MultiColorBandChannelBeo(ModelManager& mgr, const SharedPtr<Beowulf>& beow);
  virtual ~MultiColorBandChannelBeo();
protected:
  virtual void buildSubChans();
private:
  SharedPtr<Beowulf> itsBeo; // keep track of beo for use in buildSubChans()
};

// ######################################################################
//! Beowulf version of MotionChannel
class MotionChannelBeo : public MotionChannel
{
public:
  MotionChannelBeo(ModelManager& mgr, const SharedPtr<Beowulf>& beow);
  virtual ~MotionChannelBeo();
protected:
  virtual void buildSubChans();
private:
  SharedPtr<Beowulf> itsBeo; // keep track of beo for use in buildSubChans()
};


#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
