/*!@file Channels.C Feature channels, surprise version */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Rob Peters <rjpeters@klab.caltech.edu>
// $Id: ChannelsSurprise.C 3367 2004-06-17 08:16:18Z itti $
//

#include "ChannelsSurprise.H"
#include "ModelOptionDefs.H"
#include "ModelManager.H"


//! This is what we will do for computeSubmap() in SingleChannel objects

// implementation that computes surprise at full map resolution
/*#define SINGLECHANNELSURPRISECOMPUTESUBMAP \
  postProcessMap(itsSCS->getSurpriseMap(index, getRawCSmap(index)), index)*/

// Implementation where we downscale and add noise to the submap
// before we compute surprise:
#define SINGLECHANNELSURPRISECOMPUTESUBMAP \
itsSCS->getSurpriseMap(index, SingleChannel::computeSubmap(index))



// ######################################################################
// ######################################################################
// #################### SINGLE CHANNELS
// ######################################################################
// ######################################################################

// ######################################################################
// IntensityChannelSurprise member definitions:
// ######################################################################
IntensityChannelSurprise::IntensityChannelSurprise(ModelManager& mgr) :
  IntensityChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Intensity Surprise",
                                   "intensitysurprise"))
{ addSubComponent(itsSCS); }

IntensityChannelSurprise::~IntensityChannelSurprise()
{ }

Image<float> IntensityChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// ConvolveChannelSurprise member definitions:
// ######################################################################
ConvolveChannelSurprise::ConvolveChannelSurprise(ModelManager& mgr) :
  ConvolveChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Convolve Surprise",
                                   "convolvesurprise"))
{ addSubComponent(itsSCS); }

ConvolveChannelSurprise::~ConvolveChannelSurprise()
{ }

Image<float> ConvolveChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// RedGreenChannelSurprise member definitions:
// ######################################################################
RedGreenChannelSurprise::RedGreenChannelSurprise(ModelManager& mgr) :
  RedGreenChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "RedGreen Surprise",
                                   "redgreensurprise"))
{ addSubComponent(itsSCS); }

RedGreenChannelSurprise::~RedGreenChannelSurprise()
{ }

Image<float> RedGreenChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// BlueYellowChannelSurprise member definitions:
// ######################################################################
BlueYellowChannelSurprise::BlueYellowChannelSurprise(ModelManager& mgr) :
  BlueYellowChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "BlueYellow Surprise",
                                   "blueyellowsurprise"))
{ addSubComponent(itsSCS); }

BlueYellowChannelSurprise::~BlueYellowChannelSurprise()
{ }

Image<float> BlueYellowChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// SORedGreenChannelSurprise member definitions:
// ######################################################################
SORedGreenChannelSurprise::SORedGreenChannelSurprise(ModelManager& mgr) :
  SORedGreenChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "SORedGreen Surprise",
                                   "soredgreensurprise"))
{ addSubComponent(itsSCS); }

SORedGreenChannelSurprise::~SORedGreenChannelSurprise()
{ }

Image<float> SORedGreenChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// SOGreenRedChannelSurprise member definitions:
// ######################################################################
SOGreenRedChannelSurprise::SOGreenRedChannelSurprise(ModelManager& mgr) :
  SOGreenRedChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "SOGreenRed Surprise",
                                   "sogreenredsurprise"))
{ addSubComponent(itsSCS); }

SOGreenRedChannelSurprise::~SOGreenRedChannelSurprise()
{ }

Image<float> SOGreenRedChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// SOBlueYellowChannelSurprise member definitions:
// ######################################################################
SOBlueYellowChannelSurprise::SOBlueYellowChannelSurprise(ModelManager& mgr) :
  SOBlueYellowChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "SOBlueYellow Surprise",
                                   "soblueyellowsurprise"))
{ addSubComponent(itsSCS); }

SOBlueYellowChannelSurprise::~SOBlueYellowChannelSurprise()
{ }

Image<float> SOBlueYellowChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// SOYellowBlueChannelSurprise member definitions:
// ######################################################################
SOYellowBlueChannelSurprise::SOYellowBlueChannelSurprise(ModelManager& mgr) :
  SOYellowBlueChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "SOYellowBlue Surprise",
                                   "SOYellowBluesurprise"))
{ addSubComponent(itsSCS); }

SOYellowBlueChannelSurprise::~SOYellowBlueChannelSurprise()
{ }

Image<float> SOYellowBlueChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// RedChannelSurprise member definitions:
// ######################################################################
RedChannelSurprise::RedChannelSurprise(ModelManager& mgr) :
  RedChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Red Surprise",
                                   "redsurprise"))
{ addSubComponent(itsSCS); }

RedChannelSurprise::~RedChannelSurprise()
{ }

Image<float> RedChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// GreenChannelSurprise member definitions:
// ######################################################################
GreenChannelSurprise::GreenChannelSurprise(ModelManager& mgr) :
  GreenChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Green Surprise",
                                   "greensurprise"))
{ addSubComponent(itsSCS); }

GreenChannelSurprise::~GreenChannelSurprise()
{ }

Image<float> GreenChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// BlueChannelSurprise member definitions:
// ######################################################################
BlueChannelSurprise::BlueChannelSurprise(ModelManager& mgr) :
  BlueChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Blue Surprise",
                                   "bluesurprise"))
{ addSubComponent(itsSCS); }

BlueChannelSurprise::~BlueChannelSurprise()
{ }

Image<float> BlueChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// YellowChannelSurprise member definitions:
// ######################################################################
YellowChannelSurprise::YellowChannelSurprise(ModelManager& mgr) :
  YellowChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Yellow Surprise",
                                   "yellowsurprise"))
{ addSubComponent(itsSCS); }

YellowChannelSurprise::~YellowChannelSurprise()
{ }

Image<float> YellowChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// GaborChannelSurprise member definitions:
// ######################################################################
GaborChannelSurprise::GaborChannelSurprise(ModelManager& mgr,
                                           const uint oriIndex,
                                           const double ang) :
  GaborChannel(mgr, oriIndex, ang),
  itsSCS(new SingleChannelSurprise(mgr, "Gabor Surprise",
                                   "gaborsurprise"))
{ addSubComponent(itsSCS); }

GaborChannelSurprise::~GaborChannelSurprise()
{ }

Image<float> GaborChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// FlickerChannelSurprise member definitions:
// ######################################################################
FlickerChannelSurprise::FlickerChannelSurprise(ModelManager& mgr) :
  FlickerChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Flicker Surprise",
                                   "flickersurprise"))
{ addSubComponent(itsSCS); }

FlickerChannelSurprise::~FlickerChannelSurprise()
{ }

Image<float> FlickerChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// ColorBandChannelSurprise member definitions:
// ######################################################################
ColorBandChannelSurprise::ColorBandChannelSurprise(ModelManager& mgr,
                                                   const uint bandidx) :
  ColorBandChannel(mgr, bandidx),
  itsSCS(new SingleChannelSurprise(mgr, "ColorBand Surprise",
                                   "colorbandsurprise"))
{ addSubComponent(itsSCS); }

ColorBandChannelSurprise::~ColorBandChannelSurprise()
{ }

Image<float> ColorBandChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// DirectionChannelSurprise member definitions:
// ######################################################################
DirectionChannelSurprise::DirectionChannelSurprise(ModelManager& mgr,
                                                   const uint dirIndex,
                                                   const double direction,
                                                   const PyramidType type) :
  DirectionChannel(mgr, dirIndex, direction, type),
  itsSCS(new SingleChannelSurprise(mgr, "Direction Surprise",
                                   "directionsurprise"))
{ addSubComponent(itsSCS); }

DirectionChannelSurprise::~DirectionChannelSurprise()
{ }

Image<float> DirectionChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// TemplateMatchChannelSurprise member definitions:
// ######################################################################
TemplateMatchChannelSurprise::
TemplateMatchChannelSurprise(ModelManager& mgr, const Image<float>& filt) :
  TemplateMatchChannel(mgr, filt),
  itsSCS(new SingleChannelSurprise(mgr, "TemplateMatch Surprise",
                                   "templatematchsurprise"))
{ addSubComponent(itsSCS); }

TemplateMatchChannelSurprise::~TemplateMatchChannelSurprise()
{ }

Image<float> TemplateMatchChannelSurprise::
computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// PedestrianChannelSurprise member definitions:
// ######################################################################
PedestrianChannelSurprise::PedestrianChannelSurprise(ModelManager& mgr) :
  PedestrianChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Pedestrian Surprise",
                                   "pedestriansurprise"))
{ addSubComponent(itsSCS); }

PedestrianChannelSurprise::~PedestrianChannelSurprise()
{ }

Image<float> PedestrianChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// JunctionChannelSurprise member definitions:
// ######################################################################
JunctionChannelSurprise::
JunctionChannelSurprise(ModelManager& mgr, VisualCortex* vcx,
                        const VisualFeature vs,
                        const int R0, const int R1,
                        const int R2, const int R3,
                        const int R4, const int R5, 
                        const int R6, const int R7) :
  JunctionChannel(mgr, vcx, vs, R0, R1, R2, R3, R4, R5, R6, R7),
  itsSCS(new SingleChannelSurprise(mgr, "Junction Surprise",
                                   "junctionsurprise"))
{ addSubComponent(itsSCS); }

JunctionChannelSurprise::~JunctionChannelSurprise()
{ }

Image<float> JunctionChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }

// ######################################################################
// DummyChannelSurprise member definitions:
// ######################################################################
DummyChannelSurprise::
DummyChannelSurprise(ModelManager& mgr) :
  DummyChannel(mgr),
  itsSCS(new SingleChannelSurprise(mgr, "Dummy Surprise", "dummysurprise"))
{ addSubComponent(itsSCS); }

DummyChannelSurprise::~DummyChannelSurprise()
{ }

Image<float> DummyChannelSurprise::computeSubmap(const uint index) const
{ return SINGLECHANNELSURPRISECOMPUTESUBMAP; }


// ######################################################################
// ######################################################################
// #################### COMPLEX CHANNELS
// ######################################################################
// ######################################################################
// ######################################################################


// ######################################################################
// MultiConvolveChannelSurprise member definitions:
// ######################################################################
MultiConvolveChannelSurprise::MultiConvolveChannelSurprise(ModelManager& mgr) :
  MultiConvolveChannel(mgr)
{
  // kill any subchans we may have had from the base class constructor...
  removeAllSubComponents();
  setHomogeneous(true);

  // since we only support 4 subchannels, let's build and name them,
  // though we don't know what their filters are yet; we'll assign a
  // filter to them in start1():
  for (int n = 0; n < 4; n ++)
    {
      SharedPtr<ConvolveChannelSurprise>
        cc(new ConvolveChannelSurprise(*manager));
      
      // let's change our subchan's name:
      char name[100];
      sprintf(name, "Convolve%d", n); cc->setDescriptiveName(name);
      sprintf(name, "conv%d", n); cc->setTagName(name);
      
      // add cc as one of our subchannels:
      addSubChan(cc);
    }
}

// ######################################################################
MultiConvolveChannelSurprise::~MultiConvolveChannelSurprise()
{ }

// ######################################################################
// Composite Color ChannelSurprise member definitions: contains single
// and double opponent color channels
// ######################################################################
CompositeColorChannelSurprise::
CompositeColorChannelSurprise(ModelManager& mgr,
                              const double dw, const double sw):
  CompositeColorChannel(mgr, 0.0, 0.0) // do not build base subchans
{
  // let's build our member channels
  if (dw > 0.0)
    {
      uint i = addSubChan(makeShared(new ColorChannelSurprise(*manager)));
      channel(i).setTotalWeight(dw);
    }
  if (sw > 0.0)
    {
      uint i = addSubChan(makeShared(new SOColorChannelSurprise(*manager)));
      channel(i).setTotalWeight(sw);
    }
}

// ######################################################################
CompositeColorChannelSurprise::~CompositeColorChannelSurprise()
{ }

// ######################################################################
// Double Opponent ColorChannelSurprise member definitions:
// ######################################################################
ColorChannelSurprise::ColorChannelSurprise(ModelManager& mgr) :
  ColorChannel(mgr)
{
  // kill any subchans we may have had from the base class constructor...
  removeAllSubComponents();
  setHomogeneous(true);

  // instantiate Surprise channels:
  itsRGidx = addSubChan(makeShared(new RedGreenChannelSurprise(mgr)));
  itsBYidx = addSubChan(makeShared(new BlueYellowChannelSurprise(mgr)));
}

// ######################################################################
ColorChannelSurprise::~ColorChannelSurprise()
{ }

// ######################################################################
RedGreenChannelSurprise& ColorChannelSurprise::rg() const
{ return dynamic_cast<RedGreenChannelSurprise&>(subChan(itsRGidx)); }

// ######################################################################
BlueYellowChannelSurprise& ColorChannelSurprise::by() const
{ return dynamic_cast<BlueYellowChannelSurprise&>(subChan(itsBYidx)); }


// ######################################################################
// Single Opponent ColorChannelSurprise member definitions:
// ######################################################################
SOColorChannelSurprise::SOColorChannelSurprise(ModelManager& mgr) :
  SOColorChannel(mgr)
{
  // kill any subchans we may have had from the base class constructor...
  removeAllSubComponents();
  setHomogeneous(true);

  // instantiate Surprise channels:
  itsRGidx = addSubChan(makeShared(new SORedGreenChannelSurprise(mgr)));
  itsGRidx = addSubChan(makeShared(new SOGreenRedChannelSurprise(mgr)));
  itsBYidx = addSubChan(makeShared(new SOBlueYellowChannelSurprise(mgr)));
  itsYBidx = addSubChan(makeShared(new SOYellowBlueChannelSurprise(mgr)));

  // the following channels are computed to obtain the pyramids for
  // the above channels
  itsRidx = addSubChan(makeShared(new RedChannelSurprise(mgr)));
  itsGidx = addSubChan(makeShared(new GreenChannelSurprise(mgr)));
  itsBidx = addSubChan(makeShared(new BlueChannelSurprise(mgr)));
  itsYidx = addSubChan(makeShared(new YellowChannelSurprise(mgr)));
  
  // these pure color channels are not considered for saliency
  r().setTotalWeight(0.0);
  g().setTotalWeight(0.0);
  b().setTotalWeight(0.0);
  y().setTotalWeight(0.0);  
}

// ######################################################################
SOColorChannelSurprise::~SOColorChannelSurprise()
{ }

// ######################################################################
SORedGreenChannelSurprise& SOColorChannelSurprise::rg() const
{ return dynamic_cast<SORedGreenChannelSurprise&>(subChan(itsRGidx)); }

// ######################################################################
SOGreenRedChannelSurprise& SOColorChannelSurprise::gr() const
{ return dynamic_cast<SOGreenRedChannelSurprise&>(subChan(itsGRidx)); }

// ######################################################################
SOBlueYellowChannelSurprise& SOColorChannelSurprise::by() const
{ return dynamic_cast<SOBlueYellowChannelSurprise&>(subChan(itsBYidx)); }

// ######################################################################
SOYellowBlueChannelSurprise& SOColorChannelSurprise::yb() const
{ return dynamic_cast<SOYellowBlueChannelSurprise&>(subChan(itsYBidx)); }

// ######################################################################
RedChannelSurprise& SOColorChannelSurprise::r() const
{ return dynamic_cast<RedChannelSurprise&>(subChan(itsRidx)); }

// ######################################################################
GreenChannelSurprise& SOColorChannelSurprise::g() const
{ return dynamic_cast<GreenChannelSurprise&>(subChan(itsGidx)); }

// ######################################################################
BlueChannelSurprise& SOColorChannelSurprise::b() const
{ return dynamic_cast<BlueChannelSurprise&>(subChan(itsBidx)); }

// ######################################################################
YellowChannelSurprise& SOColorChannelSurprise::y() const
{ return dynamic_cast<YellowChannelSurprise&>(subChan(itsYidx)); }


// ######################################################################
// OrientationChannelSurprise member definitions:
// ######################################################################

// ######################################################################
OrientationChannelSurprise::OrientationChannelSurprise(ModelManager& mgr) :
  OrientationChannel(mgr)
{
  // let's build our channels; we may have to re-build them if
  // itsNumOrient get changed on us before we start():
  buildSubChans();
}

// ######################################################################
void OrientationChannelSurprise::buildSubChans()
{
  // kill any subchans we may have had...
  removeAllSubComponents();
  setHomogeneous(true);

  // let's instantiate our Gabor subchannels now that we know how many
  // we want. They will inherit the current values (typically
  // post-command-line parsing) of all their options as they are
  // constructed:
  LINFO("Using %d orientations spanning [0..180]deg", itsNumOrients.getVal());
  for (uint ori = 0; ori < itsNumOrients.getVal(); ++ori)
    addSubChan(makeShared
               (new GaborChannelSurprise(*manager, ori, 180.0 * double(ori) /
                                         double(itsNumOrients.getVal()))));
  // Here, we just use GaborChannelSurprise's default tag names

  // if we have been requested to export some options, let's ask it
  // again on our newly built channels:
  if (itsExportOpts != OPTEXP_NONE) exportOptions(itsExportOpts, true);
}

// ######################################################################
OrientationChannelSurprise::~OrientationChannelSurprise()
{ }

// ######################################################################
GaborChannelSurprise& OrientationChannelSurprise::gabor(const uint idx) const
{ return dynamic_cast<GaborChannelSurprise&>(subChan(idx)); }

// ######################################################################
// MultiColorBandChannelSurprise member definitions:
// ######################################################################

// ######################################################################
MultiColorBandChannelSurprise::
MultiColorBandChannelSurprise(ModelManager& mgr) :
  MultiColorBandChannel(mgr)
{
  // let's create our subchannels (may be reconfigured if itsNumBands
  // gets changed):
  buildSubChans();
}

// ######################################################################
void MultiColorBandChannelSurprise::buildSubChans()
{
  // kill any subchans we may have had...
  removeAllSubComponents();
  setHomogeneous(true);

  // let's instantiate our subchannels now that we know how many we
  // want. They will inherit the current values (typically
  // post-command-line parsing) of all their options as they are
  // constructed:
  LINFO("Using %d color bands", itsNumBands.getVal());
  for (uint i = 0; i < itsNumBands.getVal(); i ++)
    addSubChan(makeShared(new ColorBandChannelSurprise(*manager, i)));
  // Here, we just use our subchannels' default tag names

  // if we have been requested to export some options, let's ask it
  // again on our newly built channels:
  if (itsExportOpts != OPTEXP_NONE)
    ModelComponent::exportOptions(itsExportOpts, true);
}

// ######################################################################
MultiColorBandChannelSurprise::~MultiColorBandChannelSurprise()
{ }

// ######################################################################
ColorBandChannelSurprise& MultiColorBandChannelSurprise::
band(const uint idx) const
{ return dynamic_cast<ColorBandChannelSurprise&>(subChan(idx)); }


// ######################################################################
// MotionChannelSurprise member definitions:
// ######################################################################

// ######################################################################
MotionChannelSurprise::MotionChannelSurprise(ModelManager& mgr) :
  MotionChannel(mgr)
{
  // let's create our subchannels (may be reconfigured later if our
  // number of directions changes):
  buildSubChans();
}

// ######################################################################
MotionChannelSurprise::~MotionChannelSurprise()
{  }

// ######################################################################
DirectionChannelSurprise& MotionChannelSurprise::dirChan(const uint idx) const
{ return dynamic_cast<DirectionChannelSurprise&>(subChan(idx)); }

// ######################################################################
void MotionChannelSurprise::buildSubChans()
{
  // kill any subchans we may have had...
  removeAllSubComponents();
  setHomogeneous(true);

  // let's instantiate our subchannels now that we know how many we
  // want. They will inherit the current values (typically
  // post-command-line parsing) of all their options as they are
  // constructed:
  LINFO("Using %d directions spanning [0..360]deg", itsNumDirs.getVal());
  for (uint i = 0; i < itsNumDirs.getVal(); i ++)
    addSubChan(makeShared
               (new DirectionChannelSurprise(*manager, i, 360.0 * double(i) / 
                                     double(itsNumDirs.getVal()),
                                     itsPyrType.getVal())));
  // Here, we just use our subchannels' default tag names

  // if we have been requested to export some options, let's ask it
  // again on our newly built channels:
  if (itsExportOpts != OPTEXP_NONE)
    ModelComponent::exportOptions(itsExportOpts, true);
}

// ######################################################################
// L Junction ChannelSurprise member definitions:
// ######################################################################
LJunctionChannelSurprise::
LJunctionChannelSurprise(ModelManager& mgr, VisualCortex* vcx) :
  LJunctionChannel(mgr, vcx)
{
  // let's build our channels
  buildSubChans();
}

// ######################################################################
LJunctionChannelSurprise::~LJunctionChannelSurprise()
{ }

// ######################################################################
void LJunctionChannelSurprise::buildSubChans()
{
  // kill any subchans we may have had...
  removeAllSubComponents();
  setHomogeneous(true);

  LINFO("Using %d orientations spanning [0..360]deg", itsNumOrients.getVal());
  for (uint ori = 0; ori < 8; )
    {
      switch (ori)
        {
        case 0:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  1,0,1,0,0,0,0,0 )));
          break;
        case 1:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,1,0,1,0,0,0,0 )));
          break;
        case 2:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,1,0,1,0,0,0 )));
          break;
        case 3:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,1,0,1,0,0 )));
          break;
        case 4:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,0,1,0,1,0 )));
          break;
        case 5:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,0,0,1,0,1 )));
          break;
        case 6:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  1,0,0,0,0,0,1,0 )));
          break;
        case 7:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,1,0,0,0,0,0,1 )));
          break;
        default:
          break;
        }
      ori += 8 / itsNumOrients.getVal();
    }
}

// ######################################################################
// T Junction ChannelSurprise member definitions:
// ######################################################################

TJunctionChannelSurprise::
TJunctionChannelSurprise(ModelManager& mgr, VisualCortex* vcx) :
  TJunctionChannel(mgr, vcx)
{
  // let's build our channels
  buildSubChans();
}
// ######################################################################
TJunctionChannelSurprise::~TJunctionChannelSurprise()
{ }

// ######################################################################
void TJunctionChannelSurprise::buildSubChans()
{
  // kill any subchans we may have had...
  removeAllSubComponents();
  setHomogeneous(true);

  LINFO("Using %d orientations spanning [0..360]deg", itsNumOrients.getVal());
  for (uint ori = 0; ori < 8; )
    {
      switch (ori)
        {
        case 0:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  1,0,0,0,1,0,1,0 )));
          break;
        case 1:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,1,0,0,0,1,0,1 )));
          break;
        case 2:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  1,0,1,0,0,0,1,0 )));
          break;
        case 3:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,1,0,1,0,0,0,1 )));
          break;
        case 4:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  1,0,1,0,1,0,0,0 )));
          break;
        case 5:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,1,0,1,0,1,0,0 )));
          break;
        case 6:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,1,0,1,0,1,0 )));
          break;
        case 7:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,1,0,1,0,1 )));
          break;
        default:
          break;
        }
      ori += 8 / itsNumOrients.getVal();
    }
}

// ######################################################################
// X Junction ChannelSurprise member definitions:
// ######################################################################
XJunctionChannelSurprise::
XJunctionChannelSurprise(ModelManager& mgr, VisualCortex* vcx) :
  XJunctionChannel(mgr, vcx)
{
  // let's build our channels
  buildSubChans();
}

// ######################################################################
XJunctionChannelSurprise::~XJunctionChannelSurprise()
{ }

// ######################################################################
void XJunctionChannelSurprise::buildSubChans()
{
  // kill any subchans we may have had...
  removeAllSubComponents();
  setHomogeneous(true);

  LINFO("Using %d orientations spanning [0..90]deg", itsNumOrients.getVal());
  if (itsNumOrients.getVal() == 1)
    addSubChan(makeShared
               (new JunctionChannelSurprise(*manager, itsVCX, visualFeature(),
                                            1,0,1,0,1,0,1,0)));
  else if (itsNumOrients.getVal() == 2)
    addSubChan(makeShared
               (new JunctionChannelSurprise(*manager, itsVCX, visualFeature(),
                                            0,1,0,1,0,1,0,1)));
}

// ######################################################################
// End point ChannelSurprise member definitions:
// ######################################################################
EndPointChannelSurprise::
EndPointChannelSurprise(ModelManager& mgr, VisualCortex* vcx) :
  EndPointChannel(mgr, vcx)
{
  // let's build our channels
  buildSubChans();
}

// ######################################################################
EndPointChannelSurprise::~EndPointChannelSurprise()
{ }

// ######################################################################
void EndPointChannelSurprise::buildSubChans()
{
  // kill any subchans we may have had...
  removeAllSubComponents();
  setHomogeneous(true);

  LINFO("Using %d orientations spanning [0..360]deg", itsNumOrients.getVal());
  for (uint ori = 0; ori < 8; )
    {
      switch (ori)
        {
        case 0:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  1,0,0,0,0,0,0,0 )));
          break;
        case 1:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,1,0,0,0,0,0,0 )));
          break;
        case 2:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,1,0,0,0,0,0 )));
          break;
        case 3:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,1,0,0,0,0 )));
          break;
        case 4:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,0,1,0,0,0 )));
          break;
        case 5:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,0,0,1,0,0 )));
          break;
        case 6:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,0,0,0,1,0 )));
          break;
        case 7:
          addSubChan(makeShared
                     (new JunctionChannelSurprise(*manager, itsVCX,
                                                  visualFeature(),
                                                  0,0,0,0,0,0,0,1 )));
          break;
        default:
          break;
        }
      ori += 8 / itsNumOrients.getVal();
    }
}

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */

