/*!@file ChannelsSurprise.H Feature channels, surprise version */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Rob Peters <rjpeters@klab.caltech.edu>
// $Id: ChannelsSurprise.H 3367 2004-06-17 08:16:18Z itti $
//

#ifndef CHANNELSSURPRISE_H_DEFINED
#define CHANNELSSURPRISE_H_DEFINED

#include "Channels.H"
#include "SingleChannelSurprise.H"

// ######################################################################
// ######################################################################
// #################### SINGLE CHANNELS
// ######################################################################
// ######################################################################

// ######################################################################
//! An intensity channel.
class IntensityChannelSurprise : public IntensityChannel
{
public:
  //! Construct with standard params.
  IntensityChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~IntensityChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A generic convolution channel.
class ConvolveChannelSurprise : public ConvolveChannel
{
public:
  //! Construct with standard params. Will need to set a filter before using
  ConvolveChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~ConvolveChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A red-green double-opponent color channel.
class RedGreenChannelSurprise : public RedGreenChannel
{
public:
  //! Construct with standard params.
  RedGreenChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~RedGreenChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A blue-yellow double-opponent color channel.
class BlueYellowChannelSurprise : public BlueYellowChannel
{
public:
  //! Construct with standard params.
  BlueYellowChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~BlueYellowChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! Single Opponent RedGreenChannelSurprise member definitions:
class SORedGreenChannelSurprise: public SORedGreenChannel
{
public:
  //! Construct with standard params.
  SORedGreenChannelSurprise(ModelManager& mgr);
 
  //! Destructor
  virtual ~SORedGreenChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! Single Opponent GreenRedChannelSurprise member definitions:
class SOGreenRedChannelSurprise: public SOGreenRedChannel
{
public:
  //! Construct with standard params.
  SOGreenRedChannelSurprise(ModelManager& mgr);
 
  //! Destructor
  virtual ~SOGreenRedChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! Single Opponent BlueYellowChannelSurprise member definitions:
class SOBlueYellowChannelSurprise: public SOBlueYellowChannel
{
public:
  //! Construct with standard params.
  SOBlueYellowChannelSurprise(ModelManager& mgr);
 
  //! Destructor
  virtual ~SOBlueYellowChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! Single Opponent YellowBlueChannelSurprise member definitions:
class SOYellowBlueChannelSurprise: public SOYellowBlueChannel
{
public:
  //! Construct with standard params.
  SOYellowBlueChannelSurprise(ModelManager& mgr);
 
  //! Destructor
  virtual ~SOYellowBlueChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A red color channel.
class RedChannelSurprise : public RedChannel
{
public:
  //! Construct with standard params.
  RedChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~RedChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A green color channel.
class GreenChannelSurprise : public GreenChannel
{
public:
  //! Construct with standard params.
  GreenChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~GreenChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A blue color channel.
class BlueChannelSurprise : public BlueChannel
{
public:
  //! Construct with standard params.
  BlueChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~BlueChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A yellow double-opponent color channel.
class YellowChannelSurprise : public YellowChannel
{
public:
  //! Construct with standard params.
  YellowChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~YellowChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! An orientation-tuned gabor channel.
class GaborChannelSurprise : public GaborChannel
{
public:
  //! Construct with standard params plus an orientation index for this object.
  GaborChannelSurprise(ModelManager& mgr, const uint oriIndex,
                       const double ang);

  //! Destructor
  virtual ~GaborChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A temporal flicker channel.
class FlickerChannelSurprise : public FlickerChannel
{
public:
  //! Construct with standard specs.
  FlickerChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~FlickerChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A color band channel
class ColorBandChannelSurprise : public ColorBandChannel
{
public:
  //! Construct with standard params.
  ColorBandChannelSurprise(ModelManager& mgr, const uint bandidx);

  //! Destructor
  virtual ~ColorBandChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! a motion sensitive channel with direction selectivity
class DirectionChannelSurprise : public DirectionChannel
{
public:
  //! Constructor with some parameters
  DirectionChannelSurprise(ModelManager& mgr, const uint dirIndex,
                           const double direction, const PyramidType type);
 
  //! Destructor
  virtual ~DirectionChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! Convolves each level of its pyramid with a feature template.
class TemplateMatchChannelSurprise : public TemplateMatchChannel
{
public:
  //! Construct with the standard params, plus a filter specification.
  TemplateMatchChannelSurprise(ModelManager& mgr, const Image<float>& filt);

  //! Virtual destructor.
  virtual ~TemplateMatchChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

// ######################################################################
//! A template-matching channel for detecting pedestrians.
class PedestrianChannelSurprise : public PedestrianChannel
{
public:
  //! Constructor
  PedestrianChannelSurprise(ModelManager& mgr);

  //! Virtual destructor.
  virtual ~PedestrianChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

//######################################################################
//! A junction channel.
class JunctionChannelSurprise : public JunctionChannel
{
public:
  //! Constructor
  JunctionChannelSurprise(ModelManager& mgr, VisualCortex* vcx,
                          const VisualFeature vs = UNKNOWN,
                          const int R0 = 0,  const int R1 = 0,
                          const int R2 = 0,  const int R3 = 0,
                          const int R4 = 0,  const int R5 = 0, 
                          const int R6 = 0,  const int R7 = 0 );

  //! Destructor
  virtual ~JunctionChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};

//######################################################################
//! A dummy channel.
class DummyChannelSurprise : public DummyChannel
{
public:
  //! Constructor
  DummyChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~DummyChannelSurprise();

  //! computeSubmap() is special and will return surprise
  virtual Image<float> computeSubmap(const uint index) const;

protected:
  SharedPtr<SingleChannelSurprise> itsSCS; //!< our surprise computer
};


// ######################################################################
// ######################################################################
// #################### COMPLEX CHANNELS
// ######################################################################
// ######################################################################
// ######################################################################

// ######################################################################
//! A generic multi-convolution channel
class MultiConvolveChannelSurprise : public MultiConvolveChannel
{
public:
  //! Construct with standard params
  MultiConvolveChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~MultiConvolveChannelSurprise();
};

//######################################################################
//! A composite color channel class that contains single and double opp
class CompositeColorChannelSurprise : public CompositeColorChannel
{
public:
  //! Construct with standard params
  CompositeColorChannelSurprise(ModelManager& mgr,
                                const double dw = 1.0, 
                                const double sw = 1.0);

  //! Destructor
  virtual ~CompositeColorChannelSurprise();
};

// ######################################################################
//! A double opponent color channel class that combines r/g, b/y subchannels.
class ColorChannelSurprise : public ColorChannel
{
public:
  //! Construct with standard params.
  ColorChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~ColorChannelSurprise();

  //! Get access the RedGreenChannel subchannel.
  virtual RedGreenChannelSurprise& rg() const;

  //! Get access to the BlueYellowChannel subchannel.
  virtual BlueYellowChannelSurprise& by() const;
};

//######################################################################
//! A single opponent color channel class that combines r/g, g/r, b/y, etc
class SOColorChannelSurprise : public SOColorChannel
{
public:
  //! Construct with standard params.
  SOColorChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~SOColorChannelSurprise();

  //! Get access the single opponent RedGreenChannelSurprise subchannel.
  virtual SORedGreenChannelSurprise& rg() const;

  //! Get access the single opponent GreenRedChannelSurprise subchannel.
  virtual SOGreenRedChannelSurprise& gr() const;

  //! Get access to the single opponent BlueYellowChannelSurprise subchannel.
  virtual SOBlueYellowChannelSurprise& by() const;

  //! Get access to the single opponent YellowBlueChannelSurprise subchannel.
  virtual SOYellowBlueChannelSurprise& yb() const;

  //! Get access the RedChannelSurprise subchannel.
  virtual RedChannelSurprise& r() const;

  //! Get access the  GreenChannelSurprise subchannel.
  virtual GreenChannelSurprise& g() const;

  //! Get access to the BlueChannelSurprise subchannel.
  virtual BlueChannelSurprise& b() const;

  //! Get access to the YellowChannelSurprise subchannel.
  virtual YellowChannelSurprise& y() const;
};

//######################################################################
//! A composite channel containing a set of orientation-tuned gabor channels.
class OrientationChannelSurprise : public OrientationChannel
{
public:
  //! Construct with standard params
  OrientationChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~OrientationChannelSurprise();

  //! Wraps ComplexChannel's subChan() and casts to GaborChannelSurprise.
  virtual GaborChannelSurprise& gabor(const uint idx) const;

protected:
  //! (re-)build our subchannels
  virtual void buildSubChans();
};


// ######################################################################
//! A composite channel containing a set of color-band channels
class MultiColorBandChannelSurprise : public MultiColorBandChannel
{
public:
  //! Construct with standard params
  MultiColorBandChannelSurprise(ModelManager& mgr);

  //! Destructor
  virtual ~MultiColorBandChannelSurprise();

  //! Wraps ComplexChannel's subChan() and casts to ColorBandChannelSurprise
  virtual ColorBandChannelSurprise& band(const uint idx) const;

protected:
  //! (re-)build our subchannels
  virtual void buildSubChans();
};

// ######################################################################
//! A composite channel containing a set of direction channels
class MotionChannelSurprise : public MotionChannel
{
public:
  //! Construct
  MotionChannelSurprise(ModelManager& mgr);

  //! destructor
  virtual ~MotionChannelSurprise();

  //! returns a specifc DirectionChannelSurprise
  virtual DirectionChannelSurprise& dirChan(const uint idx) const;

protected:
  //! (re-)build our subchannels
  virtual void buildSubChans();
};

//######################################################################
//! An L junction channel that detects L junctions at different orientations
class LJunctionChannelSurprise :  public LJunctionChannel
{
public:
  //! Construct with standard params.
  LJunctionChannelSurprise(ModelManager& mgr, VisualCortex* vcx);

  //! Destructor
  virtual ~LJunctionChannelSurprise();

protected:
  //! (re-)build our subchannels
  virtual void buildSubChans();
};

//######################################################################
//! A T junction channel that detects T junctions at different orientations
class TJunctionChannelSurprise :  public TJunctionChannel
{
public:
  //! Construct with standard params.
  TJunctionChannelSurprise(ModelManager& mgr, VisualCortex* vcx);

  //! Destructor
  virtual ~TJunctionChannelSurprise();

protected:
  //! (re-)build our subchannels
  virtual void buildSubChans();
};

//######################################################################
//! A X junction channel that detects crosses at different orientations
class XJunctionChannelSurprise :  public XJunctionChannel
{
public:
  //! Construct with standard params.
  XJunctionChannelSurprise(ModelManager& mgr, VisualCortex* vcx);

  //! Destructor
  virtual ~XJunctionChannelSurprise();

protected:
  //! (re-)build our subchannels
  virtual void buildSubChans();
};

//######################################################################
//! An end point detector channel that detects line ends at different
//orientations
class EndPointChannelSurprise :  public EndPointChannel
{
public:
  //! Construct with standard params.
  EndPointChannelSurprise(ModelManager& mgr, VisualCortex* vcx);

  //! Destructor
  virtual ~EndPointChannelSurprise();

protected:
  //! (re-)build our subchannels
  virtual void buildSubChans();
};


#endif // !CHANNELS_H_DEFINED

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
