/*!@file FrameGrabberMode.H definitions of possible image grabbing modes */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: FrameGrabberMode.H 2493 2003-04-29 08:42:03Z itti $
//

#ifndef FRAMEGRABBERMODE_H_DEFINED
#define FRAMEGRABBERMODE_H_DEFINED


//! FrameGrabber grab modes. Not all may be supported by various hardware.
/*! See FrameGrabber.H for functions using those modes. If you add
  modes here, be sure to update the string descriptions in the
  frameGrabberModeName() function below. */
enum FrameGrabberMode {
  GMGREY    = 0,
  GMRAW     = 1,
  GMRGB555  = 2,
  GMRGB565  = 3,
  GMRGB24   = 4,
  GMRGB32   = 5,
  GMYUYV    = 6,  // I think these two are redundant/useless
  GMUYVY    = 7,  // I think these two are redundant/useless
  GMYUV444  = 8,
  GMYUV422  = 9,
  GMYUV411  = 10, 
  GMYUV420  = 11, // does this exist?
  GMYUV410  = 12, // does this exist?
  GMYUV444P = 13,
  GMYUV422P = 14,
  GMYUV411P = 15,
  GMYUV420P = 16,
  GMYUV410P = 17,
  GMAUTO          // auto selection of best mode
};

//! number of known frame grabber modes
#define NBFRAMEGRABBERMODES 19

//! Text descriptions of the various known grabmodes:
inline const char *frameGrabberModeName(const FrameGrabberMode val)
{
  static const char *FrameGrabberModeString[] = {
    "GREY", "RAW", "RGB555", "RGB565", "RGB24", "RGB32", "YUYV", "UYVY",
    "YUV444", "YUV422", "YUV411", "YUV420", "YUV410", "YUV444P", "YUV422P",
    "YUV411P", "YUV420P", "YUV410P", "Auto" };
  return FrameGrabberModeString[val];
}

#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
