/*!@file FrameRange.H A range of frames at a given inter-frame delay */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: FrameRange.H 2493 2003-04-29 08:42:03Z itti $
//

#ifndef FRAMERANGE_H_DEFINED
#define FRAMERANGE_H_DEFINED

//! Specification of a range of frames with given inter-frame delay (in ms)
class FrameRange {
public:
  //! Default constructor (first=last=0, delay=0.0F)
  inline FrameRange();

  //! Initialized constructor
  inline FrameRange(const int thefirst, const int thelast,
                    const float thedelay);

  //! Is teh series multiframe (last != first)?
  inline bool isMultiframe() const;

  int first;   //!< number of first frame
  int last;    //!< number of last frame
  float delay; //!< inter-frame delay in ms
};

// ######################################################################
// #################### INLINED METHODS
// ######################################################################
inline FrameRange::FrameRange() :
  first(0), last(0), delay(0.0F)
{ }

// ######################################################################
inline FrameRange::FrameRange(const int thefirst, const int thelast,
                              const float thedelay) :
  first(thefirst), last(thelast), delay(thedelay)
{ }

// ######################################################################
inline bool FrameRange::isMultiframe() const
{ return first != last; }

#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
