/*!@file GistEstimator.H extract estimated gist
         using available features of the image                          */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Christian Siagian <siagian@usc.edu>
// $Id: GistEstimator.H 4198 2005-02-15 02:35:14Z siagian $
//

// ######################################################################
/*! Extract gist of image                                               */

#ifndef GISTESTIMATOR_H_DEFINED
#define GISTESTIMATOR_H_DEFINED

#include "ModelComponent.H"
#include "SharedPtr.H"
#include "Image.H"
#include "VisualCortex.H"
#include "XWindow.H"
#include "XWinManaged.H"

//! implementation of the gist estimation procedure
//! yet to be published
class GistEstimator : public ModelComponent
{
public:

  //! Constructor
  /*! @param brain this is the hook for the class to obtain its information
    on channels, submaps etc. */
  GistEstimator(ModelManager& mgr,
		const std::string& descrName = "Gist Estimator",
		const std::string& tagName = "gistestimator");

  //! Destructor
  virtual ~GistEstimator();

  //! Export available options
  virtual void exportOptions(const int whatToExport, 
		  	     const bool recurse = true);  
  
  //! set the Visual Cortex if it was not given at construction
  void setVC(const SharedPtr<VisualCortex>& inVC);

  //! procedure at the start of the process
  //  void start1();
    
  //! compute gist features
  void compute();

  //! return the gist feature values
  void getGist(double* retV);

  //! get the histogram of the passed in gist feature vector
  Image<float> getHistogram(double* arr, float max, int sqSize);

private:

  // get the gist features
  void getFeatureVector();

  // compute the sub-sum pyramid values
  void getSubSum(double* res, Image<float> img, int printRes);

  // a hook to the VisualCortex
  // from which GistEstimator obtains all the raw filtered data
  SharedPtr<VisualCortex> vc;

  // gist feature vector
  double* gistV;

  // temporary feature vector storage
  double* tempV;

  //  CloseButtonListener wList;

};

#endif


// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
