/*!@file GistEstimatorConfigurator.C
  future expansions for run-time options                                */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Christian Siagian <siagian@usc.edu>
//

#include "GistEstimatorConfigurator.H"
#include "GistEstimator.H"
#include "ModelManager.H"
#include "ModelOptionDefs.H"

// ######################################################################
GistEstimatorConfigurator::
GistEstimatorConfigurator(ModelManager& mgr,
			   const std::string& descrName,
			   const std::string& tagName) :
  ModelComponent(mgr, descrName, tagName),
  itsGEtype(this, "GistEstimatorType", "None"),
  itsGE(NULL), itsWhatToExport(OPTEXP_NONE)
{  
  itsGE.reset(new GistEstimator(*manager, "Gist Estimator", 
				 "GistEstimator"));
}

// ######################################################################
GistEstimatorConfigurator::~GistEstimatorConfigurator()
{  }

// ######################################################################
void GistEstimatorConfigurator::exportOptions(const int whatToExport,
					       const bool recurse)
{
  // keep a copy of whatToExport so that we can let our controller
  // know once we instantiate it:
  itsWhatToExport = whatToExport;

  // let's first do our exports:
  if (whatToExport & OPTEXP_CORE)
    manager->requestOption(this, itsGEtype);

  // the base class will propagate this call to our subcomponents:
  ModelComponent::exportOptions(recurse);
}

// ######################################################################
SharedPtr<GistEstimator> GistEstimatorConfigurator::getGE() const
{ return itsGE; }

// ######################################################################
bool GistEstimatorConfigurator::
setModelParamString(const std::string& name,
                    const std::string& value,
                    const bool recurse,
                    const bool warn)
{
  // accept the parameter change:
  bool ret = ModelComponent::setModelParamString(name, value, recurse, warn);

  // was that a change of our baby's name?
  if (name.compare(itsGEtype.getName()) == 0) {
    // if we had one, let's unregister it and destroy it; upon
    // destruction, it will unexport its command-line options:
    if (itsGE.get()) { removeSubComponent(itsGE); itsGE.reset(NULL); }

    // instantiate a controller of the appropriate type:
    if (value.compare("None") == 0)
      itsGE.reset(NULL);
    else if (value.compare("Std") == 0)
      itsGE.reset(new GistEstimator(*manager));
    else
      LFATAL("Unknown GistEstimator type %s -- ABORT", value.c_str());

    // add our baby as a subcomponent of us so that it will become
    // linked to the manager through us (hopefully we are registered
    // with the manager), which in turn will allow it to export its
    // command-line options and get configured:
    if (itsGE.get()) {
      addSubComponent(itsGE);

      // tell it to export its options:
      itsGE->exportOptions(itsWhatToExport, true);
    }

    // some info message:
    LINFO("Selected GE of type %s", value.c_str());
  }
  return ret;
}


// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
