/*!@file Histogram.C Histogram */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: James Bonaiuto <bonaiuto@usc.edu>
// $Id: Histogram.C 3800 2004-10-07 02:08:16Z jbonaiuto $
//

#include "Histogram.H"

// #######################################################################
// #######################################################################
// ########## Histogram implementation
// #######################################################################
// #######################################################################

// #######################################################################
Histogram::Histogram(int size)
{
  histogram = std::vector<float>(size);
}

// #######################################################################
void Histogram::addValue( int index, float value )
{
  if((uint)index < histogram.size())
    histogram[index] += value;
  else
    histogram.push_back(value);
}

// #######################################################################
float Histogram::getValue(int index)
{
  if ((uint)index < histogram.size())
    return histogram[index];
  else
    return 0.0f;
}

// #######################################################################
void Histogram::smooth()
{
  float vect[ histogram.size() ];
  uint n;

  for( n = 0 ; n < histogram.size() ; n++ )
    {
      float val0 = histogram[ (n-1+histogram.size()) % histogram.size() ];
      float val1 = histogram[ (n  +histogram.size()) % histogram.size() ];
      float val2 = histogram[ (n+1+histogram.size()) % histogram.size() ];

      vect[n] = (val0+val1+val2)/3.0f;
    }
  for( n = 0 ; n < histogram.size() ; n++ )
    histogram[n] = vect[n];
}

// #######################################################################
float Histogram::findMax()
{
  float max = -1e10;
  uint n;
  for( n = 0 ; n < histogram.size() ; n++ )
    if (histogram[n] > max )
      max = histogram[n];
  return max;
}

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
