/*!@file IORtypes.H Definition of the inhibitionof-return (IOR) types */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: IORtypes.H 2493 2003-04-29 08:42:03Z itti $
//

#ifndef IORTYPES_H_DEFINED
#define IORTYPES_H_DEFINED

// ######################################################################
//! Possible types of pyramids
enum IORtype {
  IORnone       = 0,  //!< No inhibition of return at all
  IORauto       = 1,  //!< Determine from SimulationViewer
  IORdisc       = 2,  //!< A disk of the radius of the focus of attention
  IORshapeEst   = 3,  //!< ShapeEstimator-shaped
  IORshapeEstSM = 4,  //!< ShapeEstimator-shaped, based on saliency map
  IORshapeEstCM = 5   //!< ShapeEstimator-shaped, based on conspicuity map
  // if you add a new type here, also update the names in the function below!
};

//! number of known PyramidTypes:
#define NBIORTYPES 6

inline const char* iorTypeName(const IORtype p)
{
  static const char n[NBIORTYPES][11] = {
    "None", "Auto", "Disc", "ShapeEst", "ShapeEstSM", "ShapeEstCM"
  };
  return n[int(p)];
}


#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
