/*!@file free functions for drawing on images
 */
// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2002   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; filed July 23, 2001, following provisional applications     //
// No. 60/274,674 filed March 8, 2001 and 60/288,724 filed May 4, 2001).//
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Dirk Walther <walther@caltech.edu>
// $Id: Image_DrawOps.C 3275 2004-05-14 06:06:47Z mundhenk $
//

#include "Image.H"
#include "Image_All.H"
#include "Point2D.H"
#include "Pixels.H"

// ######################################################################
void drawTraj(Image< PixRGB<byte> >& img, std::vector<Point2D> foas)
{
  for (uint i = 0; i < foas.size(); i++)
    {
      img.drawCircle(foas[i],10,PixRGB<byte>(255,255,0), 1);
      if (i > 0) img.drawArrow(foas[i-1], foas[i], PixRGB<byte>(255,0,0), 1);
    }
}

// ######################################################################
Image< PixRGB<byte> > colGreyCombo(const Image< PixRGB<byte> >& colimg,
                                   const Image<float>& bwimg,
                                   const bool xcombo)
{
  // will convert and clamp to byte value range as necessary:
  Image<byte> tmp = rescale(bwimg, colimg.getWidth(), colimg.getHeight());
  Image< PixRGB<byte> > tmp2 = makeRGB(tmp, tmp, tmp);

  if (xcombo) return concatX(colimg, tmp2); else return concatY(colimg, tmp2);
}

// ######################################################################
Image< PixRGB<byte> > colColCombo(const Image< PixRGB<byte> >& colimg1,
                                   const Image< PixRGB<byte> >& colimg2,
                                   const bool xcombo)
{
  Image< PixRGB<byte> > tmp;
  if (xcombo)
    {
      if(colimg1.getHeight() != colimg2.getHeight())
        tmp = rescale(colimg2, colimg1.getWidth() / 2, colimg1.getHeight());
      else tmp = colimg2;
      return concatX(colimg1, tmp); 
    }
  else
    {
      if(colimg1.getWidth() != colimg2.getWidth())
        tmp = rescale(colimg2, colimg1.getWidth(), colimg1.getHeight() / 2);
      else tmp = colimg2;
      return concatY(colimg1, tmp);
    }
}
// ######################################################################
Image< PixRGB<byte> > highlightRegions(const Image< PixRGB<byte> >& img,
                                       const Image<byte>& mask,
                                       const int maxval)
{
  Image<float> tmp = chamfer34(mask);
  tmp = binaryReverse(tmp, float(maxval));
  tmp.setVal(0, 0, 0.0); 
  tmp.normalize(0.0, 1.0);
  return img * tmp;
}

// ######################################################################
Image< PixRGB<byte> > warp3Dmap(const Image< PixRGB<byte> >& img,
                                const Image<float>& hmap,
                                const float pitch, const float yaw,
                                Dims& imdims)
{
    // smooth linear interpolation of the height map:
    Image<float> ftmp = lowPass9(rescale(hmap,img.getWidth(),img.getHeight()));
    ftmp.clamp(0.0, 255.0);
    
    // first call to this fct will compute size of output:
    return warp3D(img, ftmp, pitch, yaw, 255.0, imdims);
}



// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
