/*!@file free functions for drawing on images
 */
// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2002   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; filed July 23, 2001, following provisional applications     //
// No. 60/274,674 filed March 8, 2001 and 60/288,724 filed May 4, 2001).//
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Dirk Walther <walther@caltech.edu>
// $Id: Image_DrawOps.H 3275 2004-05-14 06:06:47Z mundhenk $
//

#ifndef IMAGE_DRAWOPS_H_DEFINED
#define IMAGE_DRAWOPS_H_DEFINED

#include "saliency.H"
#include <vector>

class Point2D;
class Dims;
class Rectangle;
template <class T> class Image;
template <class T> class PixRGB;


//! draw the trajectory for the points in foas onto img
/*! Yellow circles and red arrows are drawn, connecting the points in foas.
  @param foas a vector with the foci of attention ised for drawing*/
void drawTraj(Image< PixRGB<byte> >& img, std::vector<Point2D> foas);

//! build a combo between a color and greyscale image
/*! @param colimg the color image (e.g., trajectory)
   @param bwimg the greyscale image (e.g., saliency map)
   @param xcombo will combine horizontally if true, otherwise vertically */
Image< PixRGB<byte> > colGreyCombo(const Image< PixRGB<byte> >& colimg,
                                   const Image<float>& bwimg,
                                   const bool xcombo);

//! build a combo between a color and color image
/*! @param colimg1 the color image (e.g., trajectory + saliency map)
   @param colimg2 the second color image (e.g., the scene highlighted by TRM)
   @param xcombo will combine horizontally if true, otherwise vertically */
Image< PixRGB<byte> > colColCombo(const Image< PixRGB<byte> >& colimg1,
                                   const Image< PixRGB<byte> >& colimg2,
                                  const bool xcombo);

//! highlight an image in the regions given by a mask
Image< PixRGB<byte> > highlightRegions(const Image< PixRGB<byte> >& img,
                                       const Image<byte>& mask,
                                       const int maxval = 320);

//! warp image onto height map
Image< PixRGB<byte> > warp3Dmap(const Image< PixRGB<byte> >& img,
                                const Image<float>& hmap,
                                const float pitch, const float yaw,
                                Dims& imdims);




// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */

#endif // !IMAGE_DRAWOPS_H_DEFINED
