/*!@file Image_ShapeOps.C Shape operations on Image
 */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Rob Peters <rjpeters@klab.caltech.edu>
// $Id: Image_ShapeOps.C 4194 2005-02-14 21:44:08Z mundhenk $
//

#ifndef IMAGE_SHAPEOPS_C_DEFINED
#define IMAGE_SHAPEOPS_C_DEFINED

#include "Image_ShapeOps.H"

#include "Image.H"
#include "Image_FilterOps.H"
#include "log.H"
#include "safecopy.H"

#include <algorithm>
#include <cmath>
#include <limits>

#define ADD_RGB(buf,Weight,bWeight,in) do { \
	buf->r += in->r*(Weight); \
	buf->g += in->g*(Weight); \
	buf->b += in->b*(Weight); \
	*bWeight += Weight; } while (0);

// ######################################################################
Image<float> downSizeClean(const Image<float>& src, const Dims& new_dims)
{
  if (src.getDims() == new_dims) return src;

  ASSERT(new_dims.isNonEmpty());

  Image<float> result = src;

  while (result.getWidth() > new_dims.w() * 2 &&
         result.getHeight() > new_dims.h() * 2)
    {
      result = decX(lowPassX(9, result));
      result = decY(lowPassY(9, result));
    }

  return rescale(result, new_dims);
}

// Include the explicit instantiations (color instantiations are now
// requested by using "T_or_RGB" for the template formal parameter name in
// the declarations in the .H file).
// MA #include "inst/Image_ShapeOps.I"

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */

#endif // !IMAGE_SHAPEOPS_C_DEFINED
