/*!@file InferoTemporal.H Object recognition module */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: InferoTemporal.H 4070 2004-12-02 02:56:16Z jbonaiuto $
//

#ifndef INFEROTEMPORAL_H_DEFINED
#define INFEROTEMPORAL_H_DEFINED

#include "ModelComponent.H"
#include "Keypoint.H"
#include "VisualObjectDB.H"

//! InferoTemporal
/*! Model of the Inferior Temporal Cortex utilizing SIFT object
    recognition
*/
class InferoTemporal : public ModelComponent{
public:
  //! Unitialized constructor
  InferoTemporal(ModelManager& mgr, const std::string& descrName,
                 const std::string& tagName);

  //! Destructor
  virtual ~InferoTemporal();

  //! Export available options
  virtual void exportOptions(const int whatToExport, 
		  	     const bool recurse = true);  
  
  //! Free all resources and get ready for a re- init()
  virtual void freeMem();

  //! Object recognition
  virtual void input(Image<PixRGB<byte> >& objImg, Point2D& location,
                     std::vector<double>& feature, int attnShiftNum);

  //! Returns whether or not to mask the object image before input
  virtual bool getUseAttention() const;

protected:
  ModelParam<bool> itsInitialScaleUp;             // scale the image up before
                                                  //  key extraction
  ModelParam<bool> itsUseAttention;               // attention modulated recog
  ModelParam<std::string> itsObjectDatabaseFile;  // object knowledgebase file
  ModelParam<bool> itsTrainObjectDB;              // train the database
  ModelParam<bool> itsPromptUserTrainDB;          // prompt user before 
                                                  //  training
  ModelParam<bool> itsMatchObjects;               // match new objects
  ModelParam<int> itsRecogMinMatch;               // min matches for recog
  ModelParam<float> itsRecogMinMatchPercent;      // min match % for recog
  ModelParam<std::string> itsOutputFile;          // output file name
  ModelParam<bool> itsSortObjects;                // sort objects by features
                                                  //  before matching
  ModelParam<std::string> itsSortObjectsBy;       // metric to sort objects by
  ModelParam<std::string> itsMatchingAlg;         // matching algorithm to use
  ModelParam<bool> itsSortKeys;                   // sort keys by magnitude
                                                  //  before matching
  
  VisualObjectDB objectDB;                        // object knowledgebase
  VisualObject obj;                               // current object
  
  //! (re-)initialization - load object database
  virtual void start1();

  //! destruction - save object database
  virtual void stop1();

  //! extract VisualObject from scene 
  /*! returns false if a VisualObject could not be extracted
   */
  bool computeVisualObject(const Image<float>& bwimg);  
};

#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
