/*!@file Jet.C a simple jet */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: Jet.C 3487 2004-08-05 23:58:43Z rjpeters $
//

#include "Jet.H"
#include "Image.H"
#include <vector>
#include <iostream>

// ######################################################################
// ######################################################################
template <class T>
Jet<T>::Jet(const SharedPtr<JetSpec> js) : Image<T>()
{ spec = js; resize(spec->getJetSize(), 1); }

// ######################################################################
template <class T>
Jet<T>::Jet() : Image<T>(), spec(NULL)
{ }

// ######################################################################
template <class T>
void Jet<T>::init(const SharedPtr<JetSpec> js)
{ ASSERT(spec.get() == NULL); spec = js; resize(spec->getJetSize(), 1); }

// ######################################################################
template <class T>
Jet<T>::~Jet()
{ this->freeMem(); }


// ######################################################################
#ifdef INST_BYTE
template class Jet<byte>;
#endif
#ifdef INST_INT16
template class Jet<int16>;
#endif
#ifdef INST_INT32
template class Jet<int32>;
#endif
#ifdef INST_FLOAT
template class Jet<float>;
#endif


// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
