/*!@file SimulationViewerEyeSim.H simulate an eye-tracker recording */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: SimulationViewerEyeSim.H 4182 2005-02-12 23:18:35Z itti $
//

#ifndef SIMULATIONVIEWEREYESIM_H_DEFINED
#define SIMULATIONVIEWEREYESIM_H_DEFINED

#include "SimulationViewerStd.H"
class SaccadeController;

//! Simulate an eye-tracker recording
/*! This simulation viewer has for main purpose to output a text file
that is in the same format as what we would get from a human eye
tracker device (.eye file). Thus it allows us to generate and save eye
movement scanpaths that come from the model but are in the same format
as human recordings. In turn, these scanpaths can then be compared to
actual human scanpaths. SimulationViewerEyeSim derived from
SimulationViewerStd and hence inherits all its behavior, simply adding
provision for generating and saving the .eye scanpath files. */
class SimulationViewerEyeSim : public SimulationViewerStd {
public:
  // ######################################################################
  /*! @name Constructors and destructors */
  //@{

  //! Constructor. See ModelComponent.H.
  SimulationViewerEyeSim(ModelManager& mgr,
                         const std::string& descrName =
                         "EyeSim Simulation Viewer",
                         const std::string& tagName =
                         "SimulationViewerEyeSim");

  //! Destructor
  virtual ~SimulationViewerEyeSim();

  //! Export some of our ModelParam parameters as command-line-options
  virtual void exportOptions(const int whatToExport,
                             const bool recurse = true);

  //@}

  // ######################################################################
  /*! @name Events that will trigger new visualizations */
  //@{

  //! Called every simulation time step (time is in seconds)
  /*! In SimulationViewerEyeSim, we save the scanpath file, then call
    SimulationViewerStd's evolve() */
  virtual void evolve(const double t);

  //@}

protected:
  //! Name of a calibrated eye movement file
  ModelParam<std::string> itsEyeFname;

  //! Period (in s) at which eye movement samples should be read in
  ModelParam<float> itsEyePeriod;

  //! Number of initial eye movement samples before stimulus starts
  ModelParam<int> itsEyeTrash;

  //! Get started
  virtual void start1();

  //! get stopped
  virtual void stop1();

private:
  FILE *itsOutFile;
  SharedPtr<SaccadeController> itsSC;
  int64 itsLastN;
};

#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
