/*!@file VisualCortexBeo.H a VisualCortex with SingleChannelBeo channels */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: VisualCortexBeo.H 2493 2003-04-29 08:42:03Z itti $
//

#ifndef VISUALCORTEXBEO_H_DEFINED
#define VISUALCORTEXBEO_H_DEFINED


#include "VisualCortex.H"
class Beowulf;

//! This is a derivation of VisualCortex that uses SingleChannelBeo channels
/*! The only difference with VisualCortex is in addDefaultChannels,
  which will use the Beo version of the channels rather than the
  normal one. See SingleChannelBeo.H and ChannelsBeo.H for info. At
  construction, we require that a Beowulf initialized as master is
  given. You may still add whatever channels you want to this
  VisualCortex; just if you want them to be Beo too, make sure you
  first get our Beowulf using getBeowulf(), the next available node
  using getNextAvailableNode() and then initilize your BeoChannels
  using this Beowulf and node number.

  If you are going to use a VisualCortexBeo, you should be running a
  process that implements a SingleChannelBeoServer on each slave node
  of your Beowulf. Such process is in beochannel-server.C for
  example. First start all your servers on all your server nodes,
  then, on the master, start your program that uses the
  VisualCortexBeo. It will configure the Beowulf, spawn the channels,
  and rip through your frames! */

class VisualCortexBeo : public VisualCortex
{
public:
  //! Constructor without any channels
  VisualCortexBeo(ModelManager& mgr,
                  const std::string& descrName,
                  const std::string& tagName,
                  const SharedPtr<Beowulf>& beo);

  //! Construct with default intens/color/orient/flicker/motion beo channels
  /*! This just calls addDefaultChannels(). The total channel weights
    are given here for each channel. If any is <= 0.0, the channel
    will NOT be added. */
  VisualCortexBeo(ModelManager& mgr, const std::string& descrName,
                  const std::string& tagName, const SharedPtr<Beowulf>& beo,
                  const double chanIw, const double chanCw,
                  const double chanOw, const double chanFw,
                  const double chanMw);

  //! Virtual destructor for safe inheritance.
  virtual ~VisualCortexBeo();

 //! Adds the default channels to the VisualCortexBeo object
  /*! intens/color/orient/flicker/motion beo channels. The total channel
    weights are given here for each channel. If any is <= 0.0, the
    channel will NOT be added. */
  void addDefaultChannels(const double chanIw = 1.0, const double chanCw = 1.0,
                          const double chanOw = 1.0, const double chanFw = 1.0,
                          const double chanMw = 1.0);

  //! get our Beowulf
  SharedPtr<Beowulf> getBeowulf() const;

protected:
  SharedPtr<Beowulf> itsBeo; //!< Our Beowulf (we are a Beowulf master)
};


#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
