/*!@file VisualCortexSurprise.C a VisualCortex with SingleChannelSurprise
  channels */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: VisualCortexSurprise.C 3367 2004-06-17 08:16:18Z itti $
//

#include "VisualCortexSurprise.H"
#include "ModelManager.H"
#include "ChannelsSurprise.H"

// ######################################################################
VisualCortexSurprise::VisualCortexSurprise(ModelManager& mgr,
                                           const std::string& descrName,
                                           const std::string& tagName) :
  VisualCortex(mgr, descrName, tagName)
{ }

// ######################################################################
VisualCortexSurprise::
VisualCortexSurprise(ModelManager& mgr, const std::string& descrName,
                     const std::string& tagName,
                     const double chanIw, const double chanCw,
                     const double chanOw, const double chanFw,
                     const double chanMw) :
  VisualCortex(mgr, descrName, tagName)
{
  addDefaultChannels(chanIw, chanCw, 0.0, chanOw, chanFw, chanMw, 
                     0.0, 0.0, 0.0, 0.0, 0.0);
}

// ######################################################################
VisualCortexSurprise::
VisualCortexSurprise(ModelManager& mgr, const std::string& descrName,
                     const std::string& tagName, const double chanIw,
                     const double chanCw, const double chanCow, 
                     const double chanOw, const double chanFw,
                     const double chanMw, const double chanLw,
                     const double chanTw, const double chanXw,
                     const double chanEw, const double chanDw) :
  VisualCortex(mgr, descrName, tagName)
{
  addDefaultChannels(chanIw, chanCw, chanCow, chanOw, chanFw, chanMw, 
                     chanLw, chanTw, chanXw, chanEw, chanDw);
}

// ######################################################################
VisualCortexSurprise::~VisualCortexSurprise()
{  }

// ######################################################################
void VisualCortexSurprise::addDefaultChannels(const double chanIw,
                                              const double chanCw,
                                              const double chanCow, 
                                              const double chanOw,
                                              const double chanFw,
                                              const double chanMw,
                                              const double chanLw,
                                              const double chanTw,
                                              const double chanXw,
                                              const double chanEw,
                                              const double chanDw)
{ 
  if (chanCw > 0.0)
    {
      addChannel("color",
                 makeShared(new ColorChannelSurprise(*manager)));
      channel("color").setTotalWeight(chanCw);
    }
  if (chanCow > 0.0)
    {
      addChannel("composite-color", makeShared
                 (new CompositeColorChannelSurprise(*manager)));
      channel("composite-color").setTotalWeight(chanCow);
    }
   if (chanFw > 0.0)
    {
      addChannel("flicker",
                 makeShared(new FlickerChannelSurprise(*manager)));
      channel("flicker").setTotalWeight(chanFw);
    }
  if (chanIw > 0.0)
    {
      addChannel("intensity",
                 makeShared(new IntensityChannelSurprise(*manager)));
      channel("intensity").setTotalWeight(chanIw);
    }
  if (chanOw > 0.0)
    {
      addChannel("orientation",
                 makeShared(new OrientationChannelSurprise(*manager)));
      channel("orientation").setTotalWeight(chanOw);
    }
  if (chanMw > 0.0)
    {
      addChannel("motion",
                 makeShared(new MotionChannelSurprise(*manager)));
      channel("motion").setTotalWeight(chanMw);
    }
  if (chanLw > 0.0)
    {
      addChannel("LJunction",
                 makeShared(new LJunctionChannelSurprise(*manager,this)));
      channel("LJunction").setTotalWeight(chanLw);
    }
  if (chanTw > 0.0)
    {
      addChannel("TJunction",
                 makeShared(new TJunctionChannelSurprise(*manager,this)));
      channel("TJunction").setTotalWeight(chanTw);
    }
  if (chanXw > 0.0)
    {
      addChannel("XJunction",
                 makeShared(new XJunctionChannelSurprise(*manager,this)));
      channel("XJunction").setTotalWeight(chanXw);
    }
  if (chanEw > 0.0)
    {
      addChannel("EndPoint",
                 makeShared(new EndPointChannelSurprise(*manager,this)));
      channel("EndPoint").setTotalWeight(chanEw);
    }
  if (chanDw > 0.0)
    {
      addChannel("Dummy",
                 makeShared(new DummyChannelSurprise(*manager)));
      channel("Dummy").setTotalWeight(chanDw);
    }
}

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
