/*!@file VisualCortexSurprise.H a VisualCortex with SingleChannelSurprise
  channels */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2000-2003   //
// by the University of Southern California (USC) and the iLab at USC.  //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: VisualCortexSurprise.H 3304 2004-05-20 21:27:59Z itti $
//

#ifndef VISUALCORTEXSURPRISE_H_DEFINED
#define VISUALCORTEXSURPRISE_H_DEFINED


#include "VisualCortex.H"

//! This is a derivation of VisualCortex using SingleChannelSurprise channels
/*! It computes Surprise instead of computing saliency. */

class VisualCortexSurprise : public VisualCortex
{
public:
  //! Constructor without any channels
  VisualCortexSurprise(ModelManager& mgr,
                       const std::string& descrName,
                       const std::string& tagName);
  //! Construct with default intens/color/orient/flicker/motion surprise chans
  VisualCortexSurprise(ModelManager& mgr, const std::string& descrName,
                       const std::string& tagName, const double chanIw,
                       const double chanCw, const double chanOw,
                       const double chanFw, const double chanMw);

  //! Construct with default Surprise channels
  VisualCortexSurprise(ModelManager& mgr, const std::string& descrName,
                       const std::string& tagName, const double chanIw,
                       const double chanCw, const double chanCow, 
                       const double chanOw, const double chanFw,
                       const double chanMw, const double chanLw,
                       const double chanTw, const double chanXw,
                       const double chanEw, const double chanDw);

  //! Virtual destructor for safe inheritance.
  virtual ~VisualCortexSurprise();

  //! Adds the default channels to the VisualCortexSurprise object
  virtual void addDefaultChannels(const double chanIw = 1.0,
                                  const double chanCw = 1.0,
                                  const double chanCow = 1.0,
                                  const double chanOw = 1.0,
                                  const double chanFw = 1.0,
                                  const double chanMw = 1.0,
                                  const double chanLw = 0.0,
                                  const double chanTw = 0.0,
                                  const double chanXw = 0.0,
                                  const double chanEw = 0.0,
                                  const double chanDw = 0.0);
};


#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
