//
//	VisualFeaturesAndPositions.h		This file is a part of the IKAROS project
// 				<Short description of the module>
//
//    Copyright (C) 2005 Mikael Asker
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	Created: 050217
//
//	<Additional description of the module>

#ifndef VISUALFEATURESANDPOSITIONS
#define VISUALFEATURESANDPOSITIONS

#include <vector>

#include "IKAROS.h"

#include "StdBrain.h"
#include "ModelManager.h"

using namespace std;

extern const float aNotAFrameRateValue;


class SwitchableIORBrain: public StdBrain 
{
public:
        SwitchableIORBrain(ModelManager&       mgr, 
                           const std::string&  descrName = "Brain",
	                       const std::string&  tagName = "Brain")
        : StdBrain(mgr, descrName, tagName)
        {}

        void stopIOR()
        {
            itsIORtype.setVal(IORnone);
        }

        void startIOR()
        {
            itsIORtype.setVal(IORdisc);
        }

        virtual ~SwitchableIORBrain() {}
};


class VisualFeaturesAndPositions: public Module 
{
public:

        enum {aNumberOfFeatureValues = 72};
        typedef vector<float> FeatureValueVector;
        static FeatureValueVector featureValuesInputToVector(float ** theFeatureValuesInput);

        static bool isAFeatureValueVector(const FeatureValueVector& aVector)
        {
            return aVector.size() == aNumberOfFeatureValues;
        }
        
        
        VisualFeaturesAndPositions(char* theName, Parameter* theParameters);
        virtual ~VisualFeaturesAndPositions();
        
        static Module*  Create(char* theName, Parameter* theParameters);

        void		SetSizes();
        
        void 		Init();
        void 		Tick();

private:

        static const float anEvolveTickLength;   // Seconds

        const bool iAmProcessingInRealTime;
        enum {aStillFrameRate = 30};   // Separates FOA shifts, which should be 30-70 ms apart
        float myFrameRate;   // Hertz
        const bool iAmProcessingAStillImage;
        double mySaliencyMapSpeed;

        const float myFloatToByteScaleFactor;
        
        // Declare some variables.
        int     myNoOfInputsX;
        int     myNoOfInputsY;
        int     myResolution;
        int     myReducedResolution;

        // The input and output vectors. The storage is declared and handled by
        // Ikaros so you do not have to.

        float** myInputRed;
        float** myInputGreen;
        float** myInputBlue;

        float** myFilterBiasInput;
        float** myTRMInput;
        float** myIORInput;

        float** myIntensityOutput;
        float** myColorOutput;
        float** myOrientationOutput;
        float** myFlickerOutput;
        float** myMotionOutput;
        
        float** myVCXOutput;
        float** mySMOutput;
        float** myAGMOutput;
        float** myWTAOutput;

        float** myTrajectoryRed;
        float** myTrajectoryGreen;
        float** myTrajectoryBlue;

        float** myFOAOutput;
        float** myFeatureValuesOutput;
        
        ModelManager myModelManager;
        SharedPtr<SwitchableIORBrain> myBrain;

        unsigned int myTickCounter;
        byte myInitCountLeft;
        unsigned int myFrameCounter;
        double mySaliencyMapTime;
};

#endif
