/*!@file VisualObjectDB.H Visual object database */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: 
//

#ifndef VISUALOBJECTDB_DEFINED
#define VISUALOBJECTDB_DEFINED

#include "VisualObject.H"
#include "Image.H"

using std::vector;

//! VisualObjectDatabase
class VisualObjectDB
{
public:
  //! Constructor
  VisualObjectDB();

  //! Construct from file
  VisualObjectDB(const char* filename, int sortOrder=0);

  //! Destructor
  ~VisualObjectDB();

  //! object getter
  const VisualObject& getObject(int index) const;
  
  //! object getter
  const VisualObject& getObject(char* objectName) const;

  //! add object
  bool addObject(VisualObject& obj);

  //! get number of objects in the database
  int getSize() const;

  //! save to file
  void save(const char* filename) const;

  //! find matche for the keypoints in the scene
  const VisualObjectMatch matchObject(VisualObject& obj, Point2D& loc,
                                      int recogMinMatch, 
                                      float recogMinMatchPercent,
                                      char* matchingAlg, bool sortKeys,
                                      bool sortObjects, char* sortBy);
  
  //! train the database on a new object
  void train(VisualObject& obj);

  //! filter the database based on the number of times an object has been
  /*! successfully recognized*/
  void filter(int recogThresh);

private:
  vector<VisualObject> objects;               // objects

  //! sort object db according to feature vector similarity with the given obj
  const vector<int> sortedIndices(const VisualObject& obj, const Point2D& loc,
                                  char*& sortBy) const;
};

#endif

// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */
