/*!@file saliency.H  Type definitions and macros
 */

// //////////////////////////////////////////////////////////////////// //
// The iLab Neuromorphic Vision C++ Toolkit - Copyright (C) 2001 by the //
// University of Southern California (USC) and the iLab at USC.         //
// See http://iLab.usc.edu for information about this project.          //
// //////////////////////////////////////////////////////////////////// //
// Major portions of the iLab Neuromorphic Vision Toolkit are protected //
// under the U.S. patent ``Computation of Intrinsic Perceptual Saliency //
// in Visual Environments, and Applications'' by Christof Koch and      //
// Laurent Itti, California Institute of Technology, 2001 (patent       //
// pending; application number 09/912,225 filed July 23, 2001; see      //
// http://pair.uspto.gov/cgi-bin/final/home.pl for current status).     //
// //////////////////////////////////////////////////////////////////// //
// This file is part of the iLab Neuromorphic Vision C++ Toolkit.       //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is free software; you can   //
// redistribute it and/or modify it under the terms of the GNU General  //
// Public License as published by the Free Software Foundation; either  //
// version 2 of the License, or (at your option) any later version.     //
//                                                                      //
// The iLab Neuromorphic Vision C++ Toolkit is distributed in the hope  //
// that it will be useful, but WITHOUT ANY WARRANTY; without even the   //
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU General Public License for more details.       //
//                                                                      //
// You should have received a copy of the GNU General Public License    //
// along with the iLab Neuromorphic Vision C++ Toolkit; if not, write   //
// to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,   //
// Boston, MA 02111-1307 USA.                                           //
// //////////////////////////////////////////////////////////////////// //
//
// Primary maintainer for this file: Laurent Itti <itti@usc.edu>
// $Id: saliency.H 4188 2005-02-13 07:01:15Z itti $
//

#ifndef SALIENCY_H_DEFINED
#define SALIENCY_H_DEFINED

// ######################################################################
// assertions

// check to see if ASSERT is already defined elsewhere
// (assuming they would have the same functionality)

#ifndef ASSERT

#include <assert.h>

//! to have some low-level checking, set ASSERT() to assert():
//#define ASSERT(x) /*  x  */
#define ASSERT(x) assert(x)

#endif

// This is now necessary on linux to have M_PI, M_LN2 and so on defined by
// <math.h>.  It might be different on other systems...
#ifndef __USE_BSD
#define __USE_BSD
#endif

// ######################################################################
// type definitions

//! Makes a <i>type</i> a typedef for \a T only if \a B is true.
template <class T, bool B>
struct type_if {};

template <class T>
struct type_if<T, true> { typedef T type; };

//! A compile-time check that \a T has \a N bits.
template <class T, unsigned long N>
struct type_with_N_bits
{
  typedef typename type_if<T, sizeof(T)*8 == N>::type type;
};

// Now use the type_if and type_with_N_bits helper structs to make some
// fixed-size integer typedef's. If the stated bit-size does not match the
// given type, then we get a compile-time error.

//! 8-bit unsigned integer
typedef type_with_N_bits<unsigned char, 8>::type byte;

//! 16-bit signed integer
typedef type_with_N_bits<short, 16>::type int16;

//! 16-bit unsigned integer
typedef type_with_N_bits<unsigned short, 16>::type uint16;

//! 32-bit signed integer
typedef type_with_N_bits<int, 32>::type int32;

//! 32-bit unsigned integer
typedef type_with_N_bits<unsigned int, 32>::type uint32;

//! 64-bit signed integer
typedef type_with_N_bits<long long int, 64>::type int64;

//! 64-bit unsigned integer
typedef type_with_N_bits<unsigned long long int, 64>::type uint64;

//! Canonical unsigned int
typedef unsigned int uint;

// ######################################################################
//! A helper class for program-wide initialization
/*! This makes a ToolkitInit object get instantiated in every translation
    unit (i.e., .C file), and thus the ToolkitInit's constructor is
    guaranteed to be run before any code in that .C file. */
class ToolkitInit
{
public:
  //! Run global initialization routines if they have yet run.
  /*! fpe_enabled determines whether or not floating-point exceptions will be
      turned on. */
  ToolkitInit(bool fpe_enabled);

  //! Run global shutdown routines if no more ToolkitInit objects around.
  ~ToolkitInit();

private:
  ToolkitInit(const ToolkitInit&); // not implemented
  ToolkitInit& operator=(const ToolkitInit&); // not implemented
};

namespace
{
  // This makes a ToolkitInit object get instantiated in every translation
  // unit (i.e., .C file), and thus the ToolkitInit's constructor is
  // guaranteed to be run before any code in that .C file.
#ifdef FPEXCEPTIONSDISABLED
  static ToolkitInit toolkitInit__(false);
#else
  static ToolkitInit toolkitInit__(true);
#endif
}


// ######################################################################
/* So things look consistent in everyone's emacs... */
/* Local Variables: */
/* indent-tabs-mode: nil */
/* End: */

#endif
